#!/bin/sh

PARAMETER=
TMPDIR=/tmp
BAKDIR=/tmp/upbak
LOGDIR=/etc/cim/log
CONDIR=/etc/cim/config
UPADMDIR=/opt/UltraPath/bin
UPINSTALLLOCKFILE="/var/lock/nxup_upgrade"
LOGFILE="/scratch/log/esxupdate.log"
INSTALLRST="installrst.txt"
OUCLOG="ouc_log.txt"
OPERLOG="oper_log"
EVENTLOG="event_log"
SERIALLOG="serial_oper"
UPCONFIG="user_cfg.xml"
UPCONFIGOLD="user_cfg.xml.old"
REBOOTFLAG=0
NEEDBOOTFLAG=0
ou_noPathCheck=1
SFCBDNOTRUNINGFLAG=0
TMPRETSTRING=""
SWITCHVERSION=806052

CURRENT_PATH=`pwd`
DIR_NAME=$(dirname $0)
CURUSER=`echo $USER`

Unattend_Conf_File=""
UnattendFlag=0
waring_error_flag=0

CURPKG_IS_CERT_FLAG=0

PKG_OSNAME=""
LOW_PKG_OSNAME=""
OS_VERSION=`vmware -v | awk '{print $3}'`
if [ "$OS_VERSION" == "5.0.0" ] || [ "$OS_VERSION" == "5.1.0" ]
then
    PKG_OSNAME=ESXi5.0
    LOW_PKG_OSNAME=ESXi5.x
fi

if [ "$OS_VERSION" == "5.5.0" ]
then
    PKG_OSNAME=ESXi5.5
    LOW_PKG_OSNAME=ESXi5.x
fi

if [ "$OS_VERSION" == "6.0.0" ] || [ "$OS_VERSION" == "6.5.0" ]
then
    PKG_OSNAME=ESXi6.x
    LOW_PKG_OSNAME=ESXi6.x
fi


Log_error()
{
    DATE=`date '+%Y-%m-%d %H:%M:%S'`
    [ "${LOGFILE}" != "" ] && echo "[$DATE][ULTRAPATH][$CURUSER][ERROR][$2] $1" >> "${LOGFILE}"
}

Log_info()
{
    DATE=`date '+%Y-%m-%d %H:%M:%S'`
    [ "${LOGFILE}" != "" ] && echo "[$DATE][ULTRAPATH][$CURUSER][INFO][$2] $1" >> "${LOGFILE}"
}

TerminaLog_error()
{
    echo -e "$1"
    Log_error "$1" "$2"
}

TerminaLog_info()
{
    echo -e "$1"
    Log_info "$1" "$2"
}

exit_with_msg()
{
    rm -rf $BAKDIR
    rm -rf $UPINSTALLLOCKFILE
    TerminaLog_error "$1" "$2"
    exit_flag=$3
    if [ "${UnattendFlag}" = "1" ] && [ "$exit_flag" != "1" ] 
    then
        waring_error_flag=1
    elif [ "${UnattendFlag}" = "1" ] && [ "$exit_flag" = "1" ]
    then
        TerminaLog_error "Unattend installation will exit. Rectify the preceding errors and perform installation again." "exit_with_msg"
        exit 1
    else
        exit 1
    fi
}

WarningNotSupportCertified()
{
    if [ "${PKG_OSNAME}" == "ESXi5.0" ]
    then
        TerminaLog_info "WARNING: Because VMware stops providing signature authentication for the software package of the ESXi 5.0/ESXi 5.1 system, the software package is not certified and signed by VMware. An alarm will be reported in vCenter if an unsigned package is used." "WarningNotSupportCertified"
    fi
    return 0
}


exit_with_pkg_not_full()
{
    exit_with_msg "Error: The installation file is damaged. Obtain a new one." "exit_with_pkg_not_full"
}
 
chmodExec() 
{ 
    local ret="" 
    if [ "$1" = "" ] || [ "$2" = "" ] 
    then 
        return 1 
    fi 

    if [ "$3" != "" ] 
    then 
        chmod "$1" "$2" "$3"
        ret=$? 
        if [ ${ret} -ne 0 ] 
        then 
            Log_error "chmod $1 $2 $3 failed." "chmodExec"
            return ${ret}
        fi 
    else 
        chmod "$1" "$2"
        ret=$? 
        if [ ${ret} -ne 0 ] 
        then 
            Log_error "chmod $1 $2 failed." "chmodExec"
            return ${ret} 
        fi 
    fi 
}

CopyFileFun()
{
    cp -rf $1 $2 >> "${LOGFILE}" 2>&1
    if [ $? -ne 0 ]
    then
        exit_with_msg "Error: Copy file $1 to $2 fail, please to contact technical service!" "CopyFileFun" "1"
    fi
}

ExecuteCliRetString()
{
    $1 > /tmp/upadmret.tmp
    TMPRETSTRING=`cat /tmp/upadmret.tmp`
    rm -f /tmp/upadmret.tmp
}

GetParameter()
{
    PATH_CHK_1=`echo ${DIR_NAME}|grep '^[.]'|wc -l`
    PATH_CHK_2=`echo ${DIR_NAME}|grep '^[/]'|wc -l`
    if [ ${PATH_CHK_1} -ne 0 ]
    then
        PARAMETER=${CURRENT_PATH}/${DIR_NAME}
    elif [ ${PATH_CHK_2} -ne 0 ]
    then
        PARAMETER=${DIR_NAME}
    else
        PATH_CHK_3=`echo ${0}|grep "/"|wc -l`
        if [ ${PATH_CHK_3} -ne 0 ]
        then
            PARAMETER=${CURRENT_PATH}/${DIR_NAME}
        else
            PARAMETER=${CURRENT_PATH}
        fi
    fi
}

CheckOverlapUpgrade()
{
    if [ -f $UPINSTALLLOCKFILE ]
    then
        pid=`cat $UPINSTALLLOCKFILE`
        if [ "${pid}" != "" ]
        then
            PROCOUNT=`ps -c | awk '{print $1}' | grep -c "^[[:blank:]]*${pid}[[:blank:]]*$"`
            if [ $PROCOUNT = 1 ]
            then
                TerminaLog_error "Error: Another UltraPath installation instance is running." "CheckOverlapUpgrade"
                exit 1
            fi
        fi
    fi

    echo $$ > $UPINSTALLLOCKFILE
}

CheckUser()
{
    if [ `echo $USER` != root ]
    then
        exit_with_msg "Error: Only user root can install UltraPath." "CheckUser"
    fi
}

CheckOSType()
{
    OSTYPE=`uname`
    if [ "$OSTYPE" != "VMkernel" ]
    then
        exit_with_msg "Error: The current system is not an ESXi system." "CheckOSType"
    fi
}

CheckOSVersion()
{
    VERSION=`vmware -v | awk '{print $3}'`
    if [ "$VERSION" != "5.0.0" ] && [ "$VERSION" != "5.1.0" ] && [ "$VERSION" != "5.5.0" ] && [ "$VERSION" != "6.0.0" ] && [ "$VERSION" != "6.5.0" ]
    then
        exit_with_msg "Error: The current operating system does not support UltraPath." "CheckOSVersion"
    fi
}

CheckPackageExist()
{
    PKGCOUNTV1R8=`ls $PARAMETER | grep "^UltraPath-8.0[0-9].[0-9][0-9][0-9]-$PKG_OSNAME-offline-bundle.zip$" | wc -l`
    PKGCOUNTV2R1=`ls $PARAMETER | grep "^UltraPath-21.0[0-9].[0-9][0-9][0-9]-$PKG_OSNAME-offline-bundle.zip$" | wc -l`
    if [ $PKGCOUNTV1R8 -eq 0 ] && [ $PKGCOUNTV2R1 -eq 0 ]
    then
        exit_with_msg "Error: The installation package does not exist, please obtain a new one." "CheckPackageExist"
    fi

    if [ $PKGCOUNTV2R1 -ne 0 ]
    then
        PKGNAME=`ls $PARAMETER | grep "^UltraPath-21.0[0-9].[0-9][0-9][0-9]-$PKG_OSNAME-offline-bundle.zip$"`        
    else
        PKGNAME=`ls $PARAMETER | grep "^UltraPath-8.0[0-9].[0-9][0-9][0-9]-$PKG_OSNAME-offline-bundle.zip$"`        
    fi  
}

CheckPackageNum()
{
    PKGCOUNTV1R8=`ls $PARAMETER | grep "^UltraPath-8.0[0-9].[0-9][0-9][0-9]-$PKG_OSNAME-offline-bundle.zip$" | wc -l`
    PKGCOUNTV2R1=`ls $PARAMETER | grep "^UltraPath-21.0[0-9].[0-9][0-9][0-9]-$PKG_OSNAME-offline-bundle.zip$" | wc -l`
    if [ $PKGCOUNTV1R8 -gt 1 ] || [ $PKGCOUNTV2R1 -gt 1 ]
    then
        exit_with_msg "Error: There are a number of different installation packages in the current directory. Please make sure only one that you wish to install to be left." "CheckPackageNum"
    fi
    
    if [ $PKGCOUNTV2R1 -eq 1 ]
    then
        PKGNAME=`ls $PARAMETER | grep "^UltraPath-21.0[0-9].[0-9][0-9][0-9]-$PKG_OSNAME-offline-bundle.zip$"`
    else
        PKGNAME=`ls $PARAMETER | grep "^UltraPath-8.0[0-9].[0-9][0-9][0-9]-$PKG_OSNAME-offline-bundle.zip$"`        
    fi
}

UnzipPackage()
{
    TMPDIR_SPACE=`vdf |grep % |grep tmp | awk '{print $4}'`
    if [ ${TMPDIR_SPACE} -lt 25600 ]
    then
        exit_with_msg "Error: The available space in the /tmp directory is insufficient. A minimum of 25MB space is required." "UnzipPackage"
    fi

    unzip -o $PARAMETER/$PKGNAME -d $BAKDIR >> "${LOGFILE}" 2>&1
    if [ $? -ne 0 ]
    then
        exit_with_msg "Error: Decompression of the installation file failed." "UnzipPackage" "1"
    fi
    
    chmodExec -R 600 $BAKDIR
}

CheckCurIfCertPkg()
{
    if [ -d $BAKDIR/vib20 ] && [ -d $BAKDIR/vib20/ultrapath.cli ] && [ -d $BAKDIR/vib20/ultrapath.cim ] && [ -d $BAKDIR/vib20/ultrapath.mpp ]
    then
        CURPKG_IS_CERT_FLAG=0
    else
        VIBCOUNT=`ls $BAKDIR | grep HUAWEI_bootbank_ultrapath | wc -l`
        if [ $VIBCOUNT -ne 0 ]
        then
            CURPKG_IS_CERT_FLAG=1
        fi
    fi
}

CheckPackageFull()
{
    if [ $CURPKG_IS_CERT_FLAG -eq 0 ]
    then
        if [ ! -d $BAKDIR/vib20/ultrapath.cli ]
        then
            exit_with_pkg_not_full
        else
            CLIPKGNAME=`ls $BAKDIR/vib20/ultrapath.cli `
            if [ $CLIPKGNAME = "" ]
            then
                exit_with_pkg_not_full
            fi
        fi

        if [ ! -d $BAKDIR/vib20/ultrapath.cim ]
        then
            exit_with_pkg_not_full
        else
            CIMPKGNAME=`ls $BAKDIR/vib20/ultrapath.cim`
            if [ $CIMPKGNAME = "" ]
            then
                exit_with_pkg_not_full
            fi
        fi

        if [ ! -d $BAKDIR/vib20/ultrapath.mpp ]
        then
            exit_with_pkg_not_full
        else
            MPPPKGNAME=`ls $BAKDIR/vib20/ultrapath.mpp`
            if [ $MPPPKGNAME = "" ]
            then
                exit_with_pkg_not_full
            fi
        fi
    else
        if [ ! -f $BAKDIR/HUAWEI_bootbank_ultrapath.cli_8.0[0-9].[0-9][0-9][0-9]-00.vib ] && [ ! -f $BAKDIR/HUAWEI_bootbank_ultrapath.cli_21.0[0-9].[0-9][0-9][0-9]-00.vib ]
        then
            exit_with_pkg_not_full
        fi

        if [ ! -f $BAKDIR/HUAWEI_bootbank_ultrapath.cim_8.0[0-9].[0-9][0-9][0-9]-00.vib ] && [ ! -f $BAKDIR/HUAWEI_bootbank_ultrapath.cim_21.0[0-9].[0-9][0-9][0-9]-00.vib ]
        then
            exit_with_pkg_not_full
        fi

        if [ ! -f $BAKDIR/HUAWEI_bootbank_ultrapath.mpp_8.0[0-9].[0-9][0-9][0-9]-00.vib ] && [ ! -f $BAKDIR/HUAWEI_bootbank_ultrapath.mpp_21.0[0-9].[0-9][0-9][0-9]-00.vib ]
        then
            exit_with_pkg_not_full
        fi
    fi
    
    if [ ! -f $BAKDIR/cfg_upgrade ]
    then
        exit_with_pkg_not_full
    fi
    
    if [ ! -f $BAKDIR/user_cfg.xml ]
    then
        exit_with_pkg_not_full
    fi
    
    if [ ! -f $BAKDIR/ouc ]
    then
        exit_with_pkg_not_full
    fi

    if [ ! -d $BAKDIR/src ]
    then
        exit_with_pkg_not_full
    fi
}

CheckSystem()
{
    ETCDIR_SPACE=`vdf |grep % |grep etc | awk '{print $4}'`
    if [ ${ETCDIR_SPACE} -lt 10240 ]
    then
        exit_with_msg "Error: The available space in the /etc directory is insufficient. A minimum of 10240KB space is required." "CheckSystem"
    fi

    ROOTDIR_SPACE=`vdf |grep % |grep root | awk '{print $4}'`
    if [ ${ROOTDIR_SPACE} -lt 10240 ]
    then
        exit_with_msg "Error: The available space in the /root directory is insufficient. A minimum of 10240KB space is required." "CheckSystem"
    fi

    TMPDIR_SPACE=`vdf |grep % |grep tmp | awk '{print $4}'`
    if [ ${TMPDIR_SPACE} -lt 51200 ]
    then
        exit_with_msg "Error: The available space in the /tmp directory is insufficient. A minimum of 50M space is required." "CheckSystem"
    fi
}

CheckNXUP()
{
    NXUPINS=`esxcli software vib list | grep ultrapath`
    if [ "$NXUPINS" != "" ]
    then
        return 0
    else
        return 1
    fi
}

CreateBakDir()
{
    if [ -d $BAKDIR ]
    then
        rm -rf $BAKDIR/*
    else
        mkdir -p $BAKDIR
        chmodExec 600 $BAKDIR
    fi
}

CheckVersion()
{
    OLDVERSION=`esxcli software vib list | grep ultrapath.mpp | awk '{print $2}' | awk -F"-" '{print $1}'`
    if [ "$OLDVERSION" = "" ]
    then
        TerminaLog_error "Failed to get the old UltraPath package version." "CheckVersion"
        rm -rf $BAKDIR
        return 1
    fi  
    
    OLDVERSION=`echo ${OLDVERSION} | awk -F "." '{print $1$2$3}'`
    OLDVERSION=$(echo ${OLDVERSION})

    ExecuteCliRetString "${UPADMDIR}/upadm show version"
    OLDDRIVERVERSION=`echo "${TMPRETSTRING}" |grep "Driver" |awk '{printf $NF}'`
    if [ "${OLDDRIVERVERSION}" != "" ]
    then
        OLDVERSION=$OLDDRIVERVERSION
        OLDVERSION=`echo ${OLDVERSION} | awk -F "." '{print $1$2$3}'`
        OLDVERSION=$(echo ${OLDVERSION})
    fi

    if [ $CURPKG_IS_CERT_FLAG -eq 0 ]
    then
        NEWVERSION=`ls $BAKDIR/vib20/ultrapath.cli | awk -F"_" '{print $4}' | awk -F"-" '{print $1}'`
        if [ "$NEWVERSION" = "" ]
        then
            TerminaLog_error "Failed to get the new UltraPath package version." "CheckVersion"
            rm -rf $BAKDIR
            return 1
        fi
    else
        NEWVERSIONV1R8=`ls $BAKDIR/HUAWEI_bootbank_ultrapath.mpp_8.0[0-9].[0-9][0-9][0-9]-00.vib 2>/dev/null | awk -F"_" '{print $4}' | awk -F"-" '{print $1}'`
        NEWVERSIONV2R1=`ls $BAKDIR/HUAWEI_bootbank_ultrapath.mpp_21.0[0-9].[0-9][0-9][0-9]-00.vib 2>/dev/null | awk -F"_" '{print $4}' | awk -F"-" '{print $1}'`
        if [ "$NEWVERSIONV1R8" = "" ] && [ "$NEWVERSIONV2R1" = "" ]
        then
            TerminaLog_error "Failed to get the new UltraPath package version." "CheckVersion"
            rm -rf $BAKDIR
            return 1
        fi
	
        if [ "$NEWVERSIONV2R1" != "" ]
        then
            NEWVERSION=$NEWVERSIONV2R1
        else
            NEWVERSION=$NEWVERSIONV1R8
        fi	
    fi
	
    NEWVERSION=`echo ${NEWVERSION} | awk -F "." '{print $1$2$3}'`
    NEWVERSION=$(echo ${NEWVERSION})
    
    if [ $OLDVERSION -gt $NEWVERSION ]
    then
        TerminaLog_error "Uninstall the UltraPath of the old version before installing the UltraPath of the target version." "CheckVersion"
        rm -rf $BAKDIR
        return 1
    fi
}

BakOldFile()
{
    if [ -f $LOGDIR/$OPERLOG ]
    then
        cp -r $LOGDIR/$OPERLOG $BAKDIR  
    fi

    if [ -f $LOGDIR/$EVENTLOG ]
    then
        cp -r $LOGDIR/$EVENTLOG $BAKDIR  
    fi

    if [ -f $LOGDIR/$SERIALLOG ]
    then
        cp -r $LOGDIR/$SERIALLOG $BAKDIR        
    fi

    if [ -f $CONDIR/$UPCONFIG ]
    then
        cp -rf $CONDIR/$UPCONFIG $CONDIR/$UPCONFIGOLD
        mv $CONDIR/$UPCONFIGOLD $BAKDIR/$UPCONFIGOLD
    else
        TerminaLog_error "The configure file $CONDIR/$UPCONFIG is not exist." "BakOldFile"
        TerminaLog_error "Please install the new package." "BakOldFile"
        return 1
    fi

    return 0
}

CopyFileToNewPkg()
{
    cp -rf $BAKDIR/$OPERLOG $LOGDIR
    cp -rf $BAKDIR/$EVENTLOG $LOGDIR
    cp -rf $BAKDIR/$SERIALLOG $LOGDIR
    cp -rf $BAKDIR/$UPCONFIG $CONDIR
    cp -rf $BAKDIR/$UPCONFIGOLD $CONDIR
}

RebootAfterUpgrade()
{
    while :
    do
        echo -n "Reboot system now? (n|y) [n] : "
        Log_info "Reboot system now? (n|y) [n] : " "RebootAfterUpgrade"
        ANSWER=""
        read ANSWER
        if [ "$ANSWER" = "Y" ] || [ "$ANSWER" = "y" ] || [ "$ANSWER" = "YES" ] || [ "$ANSWER" = "Yes" ] || [ "$ANSWER" = "yes" ]
        then
            Log_info "The answer is $ANSWER,exit." "RebootAfterUpgrade"
            REBOOTFLAG=1
            reboot
            break
        elif [ "$ANSWER" = "N" ] || [ "$ANSWER" = "n" ] || [ "$ANSWER" = "NO" ] || [ "$ANSWER" = "No" ] || [ "$ANSWER" = "no" ]
        then
            Log_info "The answer is $ANSWER,exit." "RebootAfterUpgrade"
            TerminaLog_info "Please reboot system later." "RebootAfterUpgrade"
            break
        else
            TerminaLog_error "Invalid input: $ANSWER, please input again." "RebootAfterUpgrade"
        fi
    done
}

CheckWatchDog()
{
    WATCHDOG=`/etc/init.d/sfcbd-watchdog status | grep not`
    if [ "$WATCHDOG" != "" ]
    then
        return 0
    else
        return 1
    fi
}

CheckLunMapYesOrNo()
{
    TerminaLog_info "WARNING: Some LUNs which should be managed by UltraPath are already managed by NMP or other third-party MPP." "CheckLunMapYesOrNo"
    TerminaLog_info "Please delete these LUNs before installing UltraPath, or reboot system after UltraPath is installed." "CheckLunMapYesOrNo"
    if [ ${UnattendFlag} -eq 1 ]
    then
        echo  "* Are you sure that you want to continue the UltraPath installation? [y,n,?] : ${UltraPathLunConfContinue}"
        Log_info "* Are you sure that you want to continue the UltraPath installation? [y,n,?] : ${UltraPathLunConfContinue}" "CheckLunMapYesOrNo"
        if [ "${UltraPathLunConfContinue}" = "Y" ] || [ "${UltraPathLunConfContinue}" = "y" ]
        then
            NEEDBOOTFLAG=1
            echo "Unattend install configuration item 'lun_exist_continue' set to '${UltraPathLunConfContinue}'. Unattend installation will continue."
            Log_info "Unattend install configuration item 'lun_exist_continue' set to '${UltraPathLunConfContinue}'. Unattend installation will continue." "CheckLunMapYesOrNo"
            TerminaLog_info "Start to install the UltraPath..." "CheckLunMapYesOrNo"
        else
            echo "Unattend install configuration item 'lun_exist_continue' set to '${UltraPathLunConfContinue}'. Unattend installation will exit."
            Log_info "Unattend install configuration item 'lun_exist_continue' set to '${UltraPathLunConfContinue}'. Unattend installation will exit." "CheckLunMapYesOrNo"
            exit 1
        fi
    else
        echo -n "* Are you sure that you want to continue the UltraPath installation? [y,n,?] : " 
        Log_info "* Are you sure that you want to continue the UltraPath installation? [y,n,?] : " "CheckLunMapYesOrNo"
    
        while :
        do
            ANSWER=""
            read ANSWER
            if [ "$ANSWER" = "Y" ] || [ "$ANSWER" = "y" ] || [ "$ANSWER" = "YES" ] || [ "$ANSWER" = "Yes" ] || [ "$ANSWER" = "yes" ]
            then
                NEEDBOOTFLAG=1
                Log_info "The answer is $ANSWER" "CheckLunMapYesOrNo"
                TerminaLog_info "Start to install the UltraPath..." "CheckLunMapYesOrNo"
                break
            elif [ "$ANSWER" = "N" ] || [ "$ANSWER" = "n" ] || [ "$ANSWER" = "NO" ] || [ "$ANSWER" = "No" ] || [ "$ANSWER" = "no" ]
            then
                exit_with_msg "The answer is $ANSWER, exit." "CheckLunMapYesOrNo"
            else
                TerminaLog_error "        ERROR: Please enter yes or no." "CheckLunMapYesOrNo"
                echo -n "* Do you want to continue with the installation of <UltraPath> [y,n,?] : "
                Log_info "* Do you want to continue with the installation of <UltraPath> [y,n,?] : " "CheckLunMapYesOrNo"
            fi
        done
    fi
}

CheckLunMap()
{
    HUAWEILUNLIST=`esxcli storage core device list |grep -c -E "HUAWEI|HUASY"`
    if [ $HUAWEILUNLIST -ne 0 ]
    then
        return 1
    fi

    MarstorLUNLIST=`esxcli storage core device list |grep -A1 "\<Vendor: Marstor\>" |grep -c "\<Model: XSG1\>"`
    if [ $MarstorLUNLIST -ne 0 ]
    then
        return 1
    fi

    UDsafeLUNLIST=`esxcli storage core device list |grep -A1 "\<Vendor: UDsafe\>" |grep -c "\<Model: XSG1\>"`
    if [ $UDsafeLUNLIST -ne 0 ]
    then
        return 1
    fi

    SanMLUNLIST=`esxcli storage core device list |grep -A1 "\<Vendor: SanM\>" |grep -c "\<Model: XSG1\>"`
    if [ $SanMLUNLIST -ne 0 ]
    then
        return 1
    fi

    AnyStorLUNLIST=`esxcli storage core device list |grep -A1 "\<Vendor: AnyStor\>" |grep -c "\<Model: XSG1\>"`
    if [ $AnyStorLUNLIST -ne 0 ]
    then
        return 1
    fi

    SUGONLUNLIST=`esxcli storage core device list |grep -A1 "\<Vendor: SUGON\>" |grep -c "\<Model: XSG1\>"`
    if [ $SUGONLUNLIST -ne 0 ]
    then
        return 1
    fi

    NETPOSALUNLIST=`esxcli storage core device list |grep -A1 "\<Vendor: NETPOSA\>" |grep -c "\<Model: XSG1\>"`
    if [ $NETPOSALUNLIST -ne 0 ]
    then
        return 1
    fi

    return 0
}

CheckClaimruleVendor()
{
    VID_NAME=$1
    PID_NAME=$2

    VIDALLRULECOUNT=`esxcli storage core claimrule list | grep -E "\<vendor=$VID_NAME\>" | grep -c -E "model=\*"`
    if [ $VIDALLRULECOUNT -ne 0 ]
    then
        VIDALLRULENO=`esxcli storage core claimrule list | grep -E "\<vendor=$VID_NAME\>" | grep "model=\*" |grep file |awk '{print $2}'`
        if [ $VIDALLRULENO -lt 400 ]
        then
            return 1
        fi
    fi

    VIDPIDRULECOUNT=`esxcli storage core claimrule list | grep -E "\<vendor=$VID_NAME\>" | grep -c -E "\<model=$PID_NAME\>"`
    if [ $VIDPIDRULECOUNT -ne 0 ]
    then
        return 1
    fi

    return 0
}

CheckUpgradeClaimruleVendor()
{
    VID_NAME=$1
    PID_NAME=$2
    RULE_NO=$3

    VIDALLRULECOUNT=`esxcli storage core claimrule list | grep -E "\<vendor=$VID_NAME\>" |grep "file" | grep -c -E "model=\*"`
    if [ $VIDALLRULECOUNT -ne 0 ]
    then
        VIDALLRULENO=`esxcli storage core claimrule list | grep -E "\<vendor=$VID_NAME\>" |grep "file" | grep "model=\*" |sed -n '1p' |awk '{print $2}'`
        if [ $VIDALLRULENO -lt 400 ]
        then
            TerminaLog_error "Please delete the claim rule corresponding $VID_NAME vendor before upgrade UltraPath." "CheckUpgradeClaimruleVendor"
            esxcli storage core claimrule list | grep -E "Rule|\<vendor=$VID_NAME\>" | grep -E "Rule|model=\*"
            return 1
        fi
    fi

    VIDPIDRULECOUNT=`esxcli storage core claimrule list | grep -E "\<vendor=$VID_NAME\>" |grep "file" | grep -c -E "\<model=$PID_NAME\>"`
    if [ $VIDPIDRULECOUNT -ne 0 ]
    then
        VIDALLRULENO=`esxcli storage core claimrule list | grep -E "\<vendor=$VID_NAME\>" |grep "file" | grep -E "\<model=$PID_NAME\>" |sed -n '1p' |awk '{print $2}'`
        if [ $VIDALLRULENO -ne $RULE_NO ]
        then
            TerminaLog_error "Please delete the claim rule corresponding $VID_NAME vendor before upgrade UltraPath." "CheckUpgradeClaimruleVendor"
            esxcli storage core claimrule list | grep -E "Rule|\<vendor=$VID_NAME\>" | grep -E "Rule|\<model=$PID_NAME\>"
            return 1
        fi
    fi

    VIDPIDRULECOUNT=`esxcli storage core claimrule list | grep -c -E "\<$RULE_NO  file\>"`
    if [ $VIDPIDRULECOUNT -ne 0 ]
    then
        VIDALLRULENO=`esxcli storage core claimrule list | grep -E "\<vendor=$VID_NAME\>" | grep -E "\<model=$PID_NAME\>" | grep -E "MP_PLUGIN_ULTRAPATH" | grep -c -E "\<$RULE_NO  file\>"`
        if [ $VIDALLRULENO -eq 0 ]
        then
            TerminaLog_error "The claim rule which used by UltraPath is in $RULE_NO has been used. Please remove the claim rule before upgrade UltraPath." "CheckUpgradeClaimruleVendor"
            esxcli storage core claimrule list | grep -E "Rule|\<$RULE_NO  file\>|\<$RULE_NO  runtime\>"
            return 1
        fi
    fi

    return 0
}

CheckUpgradeClaimrule()
{
    CheckUpgradeClaimruleVendor Marstor XSG1 472
    if [ $? -ne 0 ]
    then 
        return 1
    fi

    CheckUpgradeClaimruleVendor UDsafe XSG1 473
    if [ $? -ne 0 ]
    then 
        return 1
    fi

    CheckUpgradeClaimruleVendor SanM XSG1 474
    if [ $? -ne 0 ]
    then 
        return 1
    fi

    CheckUpgradeClaimruleVendor AnyStor XSG1 475
    if [ $? -ne 0 ]
    then 
        return 1
    fi

    CheckUpgradeClaimruleVendor SUGON XSG1 476
    if [ $? -ne 0 ]
    then 
        return 1
    fi

    CheckUpgradeClaimruleVendor NETPOSA XSG1 477
    if [ $? -ne 0 ]
    then 
        return 1
    fi

    return 0
}

CheckClaimrule()
{
    CLAIMRULENUM=`esxcli storage core claimrule list | grep -c -E "\<4[0-6][0-9]\>|\<47[0-7]\>"`
    if [ $CLAIMRULENUM -ne 0 ]
    then 
        TerminaLog_error "The claim rule which used by UltraPath is in the range 400-477 has been used by other plugin. Please remove the claim rule before installing UltraPath." "CheckClaimrule"
        esxcli storage core claimrule list | grep -E "Rule|\<4[0-6][0-9]\>|\<47[0-7]\>"
        return 1
    fi

    CLAIMRULEVENDOR=`esxcli storage core claimrule list | grep -c -E "\<HUAWEI\>|\<HUASY\>|\<HS\>"`
    if [ $CLAIMRULEVENDOR -ne 0 ]
    then 
        TerminaLog_error "Please delete the claim rule corresponding HUAWEI vendor before installing UltraPath." "CheckClaimrule"
        esxcli storage core claimrule list | grep -E "Rule|\<HUAWEI\>|\<HUASY\>|\<HS\>"
        return 1
    fi

   CheckClaimruleVendor Marstor XSG1
    if [ $? -ne 0 ]
    then 
        TerminaLog_error "Please delete the claim rule corresponding Mars vendor before installing UltraPath." "CheckClaimrule"
        esxcli storage core claimrule list | grep -E "Rule|\<vendor=Marstor\>" | grep -E "Rule|model=\*|\<model=XSG1\>"
        return 1
    fi

    CheckClaimruleVendor UDsafe XSG1
    if [ $? -ne 0 ]
    then 
        TerminaLog_error "Please delete the claim rule corresponding UDsafe vendor before installing UltraPath." "CheckClaimrule"
        esxcli storage core claimrule list | grep -E "Rule|\<vendor=UDsafe\>" | grep -E "Rule|model=\*|\<model=XSG1\>"
        return 1
    fi

    CheckClaimruleVendor SanM XSG1
    if [ $? -ne 0 ]
    then 
        TerminaLog_error "Please delete the claim rule corresponding SUNMnet vendor before installing UltraPath." "CheckClaimrule"
        esxcli storage core claimrule list | grep -E "Rule|\<vendor=SanM\>" | grep -E "Rule|model=\*|\<model=XSG1\>"
        return 1
    fi

    CheckClaimruleVendor AnyStor XSG1
    if [ $? -ne 0 ]
    then 
        TerminaLog_error "Please delete the claim rule corresponding EISOO vendor before installing UltraPath." "CheckClaimrule"
        esxcli storage core claimrule list | grep -E "Rule|\<vendor=AnyStor\>" | grep -E "Rule|model=\*|\<model=XSG1\>"
        return 1
    fi

    CheckClaimruleVendor SUGON XSG1
    if [ $? -ne 0 ]
    then 
        TerminaLog_error "Please delete the claim rule corresponding SUGON vendor before installing UltraPath." "CheckClaimrule"
        esxcli storage core claimrule list | grep -E "Rule|\<vendor=SUGON\>" | grep -E "Rule|model=\*|\<model=XSG1\>"
        return 1
    fi

    CheckClaimruleVendor NETPOSA XSG1
    if [ $? -ne 0 ]
    then 
        TerminaLog_error "Please delete the claim rule corresponding NETPOSA vendor before installing UltraPath." "CheckClaimrule"
        esxcli storage core claimrule list | grep -E "Rule|\<vendor=NETPOSA\>" | grep -E "Rule|model=\*|\<model=XSG1\>"
        return 1
    fi

    return 0
}

RestartHostdNote()
{
    TerminaLog_info "Note: The SSD property of a disk cannot be updated after an online upgrade. Reclaim the disk or restart the host to update the SSD property."
    TerminaLog_info "Note: During the upgrade, the hostd service on ESXi hosts will be restarted and vCenter will be disconnected temporarily. This will affect the services (clone, migration, or snapshot) running on vCenter." "HotSwapUercfgXml"
    TerminaLog_info "      To avoid this situation, do not perform any management task during the upgrade, and you will be prompted to check whether there are services running on vCenter during the upgrade." "HotSwapUercfgXml"
    
    while :
    do
        echo -n "      Are there no services running on vCenter? If yes, continue with the upgrade. If no, exit the upgrade. [y,n,?]: "
        ANSWER=""
        read ANSWER
        if [ "$ANSWER" = "Y" ] || [ "$ANSWER" = "y" ] || [ "$ANSWER" = "YES" ] || [ "$ANSWER" = "Yes" ] || [ "$ANSWER" = "yes" ]
        then
            break
        elif [ "$ANSWER" = "N" ] || [ "$ANSWER" = "n" ] || [ "$ANSWER" = "NO" ] || [ "$ANSWER" = "No" ] || [ "$ANSWER" = "no" ]
        then
            exit_with_msg "The answer is $ANSWER, exit." "HotSwapUercfgXml"
        else
            TerminaLog_error "Invalid input: $ANSWER, please input again." "HotSwapUercfgXml"
        fi
    done
}

OUCFILE=ouc
OLD_PKG_NAME=""
BACKUP_PKG_DIR=/etc/UltraPath/package 
NXUP_EXT_A=nxup-ext-a 
NXUP_EXT_B=nxup-ext-b
NXUP_BASE=mp-plugin-ultrapath
CURENT_LOAD_EXT=nxup-ext-a
OLD_PKG_UNZIP=oldpkg
ETCDIR_FREE_SPACE=10240
CURRENT_VERSION=""

RESTART_HOSTD_FLAG=1
BAKPKG_FULL_FLAG=1

CreateBackupPackageDir()
{
    if [ ! -d ${BACKUP_PKG_DIR} ]
    then
        mkdir -p ${BACKUP_PKG_DIR}
        chmodExec 600 ${BACKUP_PKG_DIR}
    fi
}

CheckOnlineUpgradeParameter()
{
    while [ $# -gt 0 ]
    do
        OU_PARAM="$1"
        OU_PARAM_1=${OU_PARAM%%=*}
        OU_PARAM_2=${OU_PARAM##*=}

        case "${OU_PARAM_1}" in
            "-ndu")
                ;;
            "ndu_free_disk_space")
                INTERGER_FLAG=`expr match "${OU_PARAM_2}" "[0-9][0-9]*$"`
                if [ ${INTERGER_FLAG} -gt 0 ]
                then
                    let ETCDIR_FREE_SPACE=`expr ${OU_PARAM_2} \* 1024`
                else
                    TerminaLog_info    "Usage: sh $0 -ndu [ndu_free_disk_space=xxx]" "CheckOnlineUpgradeParameter"
                    exit_with_msg "       ndu_free_disk_space: Available disk space required by the online upgrade. xxx must be an integer." "CheckOnlineUpgradeParameter"
                fi
                ;;
            *)            
                TerminaLog_info    "Usage: sh $0 -ndu [ndu_free_disk_space=xxx]" "CheckOnlineUpgradeParameter"
                exit_with_msg "       ndu_free_disk_space: Available disk space required by the online upgrade. xxx must be an integer." "CheckOnlineUpgradeParameter"
                ;;
        esac
        shift
    done
}

CheckOnlineUpgrageSpecificationAfterOU()
{
    rm -rf $UPINSTALLLOCKFILE
    ExecuteCliRetString "${UPADMDIR}/upadm check status"
    RET=`echo "${TMPRETSTRING}" |grep -E -c "out of specifications|Specifications are being updated"`
    if [ ${RET} -gt 0 ]
    then
        TerminaLog_error "Warning: Current specifications exceed UltraPath limitations. Run the \"esxcli upadm check status\" command to check the specifications." "CheckOnlineUpgrageSpecificationAfterOU"
    fi
}

CheckOnlineUpgrageSystem()
{
    ETCDIR_SPACE=`vdf |grep % |grep etc | awk '{print $4}'`
    if [ ${ETCDIR_SPACE} -lt ${ETCDIR_FREE_SPACE} ]
    then
        exit_with_msg "Error: The available space in the /etc directory is insufficient. A minimum of ${ETCDIR_FREE_SPACE}KB space is required." "CheckOnlineUpgrageSystem"
    fi

    ROOTDIR_SPACE=`vdf |grep % |grep root | awk '{print $4}'`
    if [ ${ROOTDIR_SPACE} -lt ${ETCDIR_FREE_SPACE} ]
    then
        exit_with_msg "Error: The available space in the /root directory is insufficient. A minimum of ${ETCDIR_FREE_SPACE}KB space is required." "CheckOnlineUpgrageSystem"
    fi
}

CheckOnlineUpgradeInputParam()
{
  if [ ${ETCDIR_FREE_SPACE} -lt 10240 ]
  then
    while :
    do
        echo -n    "Note: If you modify online upgrade parameters, the online upgrade may fail. Do you want to continue? [y,n,?]: "
        ANSWER=""
        read ANSWER
        if [ "$ANSWER" = "Y" ] || [ "$ANSWER" = "y" ] || [ "$ANSWER" = "YES" ] || [ "$ANSWER" = "Yes" ] || [ "$ANSWER" = "yes" ]
        then
            break
        elif [ "$ANSWER" = "N" ] || [ "$ANSWER" = "n" ] || [ "$ANSWER" = "NO" ] || [ "$ANSWER" = "No" ] || [ "$ANSWER" = "no" ]
        then
            exit_with_msg "" "CheckOnlineUpgradeInputParam"
        else
            TerminaLog_error "Invalid input: $ANSWER, please input again." "CheckOnlineUpgradeInputParam"
        fi
    done
  fi
}

CheckOnlineUpgrageUpadm()
{
    if [ `/etc/init.d/hostd status |grep -c "hostd is running"` -ne 1 ]
    then
        exit_with_msg "Error: The installed UltraPath is not running correctly." "CheckOnlineUpgrageUpadm"
    fi
    
    if [ `ps -c |grep -c upadm` -gt 1 ]
    then
        exit_with_msg "Error: An upadm process is running. Stop the process and try again." "CheckOnlineUpgrageUpadm"
    fi
    
    if [ `esxcli software vib list |grep -c "ultrapath"` -lt 3 ]
    then
        exit_with_msg "Error: The installed UltraPath is not running correctly." "CheckOnlineUpgrageUpadm"
    fi
    
    ExecuteCliRetString "${UPADMDIR}/upadm show version"
    if [ `echo "${TMPRETSTRING}" |grep -c "Version"` -ne 2 ]
    then
        exit_with_msg "Error: The installed UltraPath is not running correctly." "CheckOnlineUpgrageUpadm"
    fi
}

CheckOnlineUpgragePackageFull()
{
    if [ -d ${BAKDIR}/${OLD_PKG_UNZIP} ]
    then
        rm -rf ${BAKDIR}/${OLD_PKG_UNZIP}/*
    else
        mkdir ${BAKDIR}/${OLD_PKG_UNZIP}
        chmodExec 600 ${BAKDIR}/${OLD_PKG_UNZIP}
    fi
    
    unzip -o ${BAKDIR}/${OLD_PKG_NAME} -d ${BAKDIR}/${OLD_PKG_UNZIP}/ >> "${LOGFILE}" 2>&1
    if [ $? -ne 0 ]
    then
        Log_info "Warn: The backup files of the installed UltraPath are not correct." "CheckOnlineUpgragePackageFull"
        return 1
    fi
    chmodExec -R 600 ${BAKDIR}/${OLD_PKG_UNZIP}
	
    if [ $CURPKG_IS_CERT_FLAG -eq 0 ]
	then
        if [ ! -d ${BAKDIR}/${OLD_PKG_UNZIP}/vib20/ultrapath.cli ]
        then
            Log_info "Warn: The backup files of the installed UltraPath are not correct." "CheckOnlineUpgragePackageFull"
            return 1
        else
            CLIPKGNAME=`ls ${BAKDIR}/${OLD_PKG_UNZIP}/vib20/ultrapath.cli `
            if [ $CLIPKGNAME = "" ]
            then
                Log_info "Warn: The backup files of the installed UltraPath are not correct." "CheckOnlineUpgragePackageFull"
                return 1
            fi
        fi

        if [ ! -d ${BAKDIR}/${OLD_PKG_UNZIP}/vib20/ultrapath.cim ]
        then
            Log_info "Warn: The backup files of the installed UltraPath are not correct." "CheckOnlineUpgragePackageFull"
            return 1
        else
            CIMPKGNAME=`ls ${BAKDIR}/${OLD_PKG_UNZIP}/vib20/ultrapath.cim`
            if [ $CIMPKGNAME = "" ]
            then
                Log_info "Warn: The backup files of the installed UltraPath are not correct." "CheckOnlineUpgragePackageFull"
                return 1
            fi
        fi

        if [ ! -d ${BAKDIR}/${OLD_PKG_UNZIP}/vib20/ultrapath.mpp ]
        then
            Log_info "Warn: The backup files of the installed UltraPath are not correct." "CheckOnlineUpgragePackageFull"
            return 1
        else
            MPPPKGNAME=`ls ${BAKDIR}/${OLD_PKG_UNZIP}/vib20/ultrapath.mpp`
            if [ $MPPPKGNAME = "" ]
            then
                Log_info "Warn: The backup files of the installed UltraPath are not correct." "CheckOnlineUpgragePackageFull"
                return 1
            fi
        fi
    else
        if [ ! -f ${BAKDIR}/${OLD_PKG_UNZIP}/HUAWEI_bootbank_ultrapath.cli_${CURRENT_VERSION}-00.vib ]
        then
            Log_info "Warn: The backup files of the installed UltraPath are not correct." "CheckOnlineUpgragePackageFull"
            return 1
        fi

        if [ ! -f ${BAKDIR}/${OLD_PKG_UNZIP}/HUAWEI_bootbank_ultrapath.cim_${CURRENT_VERSION}-00.vib ]
        then
            Log_info "Warn: The backup files of the installed UltraPath are not correct." "CheckOnlineUpgragePackageFull"
            return 1
        fi

        if [ ! -f ${BAKDIR}/${OLD_PKG_UNZIP}/HUAWEI_bootbank_ultrapath.mpp_${CURRENT_VERSION}-00.vib ]
        then
            Log_info "Warn: The backup files of the installed UltraPath are not correct." "CheckOnlineUpgragePackageFull"
            return 1
        fi
    fi
    
    if [ ! -f ${BAKDIR}/${OLD_PKG_UNZIP}/cfg_upgrade ]
    then
        Log_info "Warn: The backup files of the installed UltraPath are not correct." "CheckOnlineUpgragePackageFull"
        return 1
    fi
    
    if [ ! -f ${BAKDIR}/${OLD_PKG_UNZIP}/user_cfg.xml ]
    then
        Log_info "Warn: The backup files of the installed UltraPath are not correct." "CheckOnlineUpgragePackageFull"
        return 1
    fi
    
    if [ `ls ${BAKDIR}/${OLD_PKG_UNZIP}/src/ |grep -c ""` -lt 6 ]
    then
        Log_info "Warn: The backup files of the installed UltraPath are not correct." "CheckOnlineUpgragePackageFull"
        return 1
    fi
    
    return 0
}

CheckOnlineUpgragePackage()
{
    ExecuteCliRetString "${UPADMDIR}/upadm show version"
    CURRENT_VERSION=`echo "${TMPRETSTRING}" 2>/dev/null|grep "Software"|awk -F " " '{print $4}'`    
    if [ `echo ${OU_SUPPORT_VER} | grep -c ",${CURRENT_VERSION},"` -eq 0 ]
    then
        exit_with_msg "Error: Installed UltraPath does not support online upgrade." "CheckOnlineUpgragePackage"
    fi
    
    upVersion=`echo ${CURRENT_VERSION} | awk -F "." '{print $1$2$3}'`
    upVersion=$(echo ${upVersion})
    
    if [ $upVersion -le $SWITCHVERSION ]
    then
        OLD_PKG_count=`ls ${BACKUP_PKG_DIR} | grep "^UltraPath-${CURRENT_VERSION}-$LOW_PKG_OSNAME-offline-bundle.zip$" -c`
        if [ $OLD_PKG_count -ne 1 ]
        then
            exit_with_msg "Error: The backup files of the installed UltraPath are not correct." "CheckOnlineUpgragePackage"
        fi
        
        OLD_PKG_NAME="UltraPath-${CURRENT_VERSION}-$LOW_PKG_OSNAME-offline-bundle.zip"
    else
        OLD_PKG_count=`ls ${BACKUP_PKG_DIR} | grep "^UltraPath-${CURRENT_VERSION}-$PKG_OSNAME-offline-bundle.zip$" -c`
        if [ $OLD_PKG_count -ne 1 ]
        then
            exit_with_msg "Error: The backup files of the installed UltraPath are not correct." "CheckOnlineUpgragePackage"
        fi
        
        OLD_PKG_NAME="UltraPath-${CURRENT_VERSION}-$PKG_OSNAME-offline-bundle.zip"
    fi    

    if [ ! -f "/etc/cim/config/user_cfg.xml" ]
    then
        exit_with_msg "Error: The configuration file of the installed UltraPath does not exist." "CheckOnlineUpgragePackage"
    fi
    
    cp -rf ${BACKUP_PKG_DIR}/${OLD_PKG_NAME} ${BAKDIR}/${OLD_PKG_NAME}

    CheckOnlineUpgragePackageFull
    if [ $? -ne 0 ]
    then
        Log_info "Warn: The backup files of the installed UltraPath are not correct." "CheckOnlineUpgragePackage"
        BAKPKG_FULL_FLAG=0
    fi
}

CheckOnlineUpgragePath()
{
    local normalPathNUM=0
    local faultLogicPathNUM=0
    local disableLogicPathNUM=0
    local degradedLogicPathNUM=0
    local checkingPhysicPathNum=0
    local waitingPhysicPathNum=0
    local normalPathNumPerLun
    local faultLogicPathNumPerLun
    local disableLogicPathNumPerLun
    local degradedLogicPathNumPerLun
    local vlunID
    local oldIFS
    local vlunInfo
    local invalidInputNum
    local ret=0
    local typePara
    local cmdtypePara
    
    ExecuteCliRetString "${UPADMDIR}/upadm show vlun type=all"
    invalidInputNum=`echo "${TMPRETSTRING}" |grep -c "Invalid input"`
    if [ $invalidInputNum -gt 0 ]
    then
        typePara=""
        cmdtypePara=""
    else
        typePara="type=all"
        cmdtypePara=" -t all"
    fi

    ExecuteCliRetString "${UPADMDIR}/upadm show vlun ${typePara}"
    vlunInfo=`echo "${TMPRETSTRING}" |grep "^[[:blank:]]*[0-9]"|awk '{printf("%s ",$1)}'`
    if [ "${vlunInfo}" = "" ]
    then
       return 0
    fi
        
    allVlunNUM=`echo "${TMPRETSTRING}" |grep "^[[:blank:]]*[0-9]"|awk '{print $1}' |uniq |wc -l`
    scsidevNUM=`esxcfg-scsidevs -l |grep -c MP_PLUGIN_ULTRAPATH`
    if [ $allVlunNUM -ne $scsidevNUM ]
    then
        exit_with_msg "Error: The states of some vluns and scsidevs are not correct. Run the \"esxcli upadm check status\" and \"esxcfg-scsidevs -l\" command to check the vluns and scsidevs. Repair the vluns and try again. " "CheckOnlineUpgragePath"
    fi

    oldIFS="${IFS}"
    IFS=","
    IFS="${oldIFS}"

    for k in $vlunInfo
    do
        vlunID=$k
        ExecuteCliRetString "${UPADMDIR}/upadm show vlun id=${vlunID} ${typePara}"
        normalPathNumPerLun=`echo "${TMPRETSTRING}" |grep "Path " |awk '{print $NF}' |grep -c "^Normal$"`
        faultLogicPathNumPerLun=`echo "${TMPRETSTRING}" |grep "Path " |awk '{print $NF}' |grep -c "^Fault$"`
        disableLogicPathNumPerLun=`echo "${TMPRETSTRING}" |grep "Path " |awk '{print $NF}' |grep -c "^Disable$"`
        degradedLogicPathNumPerLun=`echo "${TMPRETSTRING}" |grep "Path " |awk '{print $NF}' |grep -c "^degradation$"`
        let "normalPathNUM=${normalPathNUM}+${normalPathNumPerLun}"
        let "faultLogicPathNUM=${faultLogicPathNumPerLun}+${faultLogicPathNUM}"
        let "disableLogicPathNUM=${disableLogicPathNumPerLun}+${disableLogicPathNUM}"
        let "degradedLogicPathNUM=${degradedLogicPathNUM}+${degradedLogicPathNumPerLun}"
        if [ "$OS_VERSION" == "5.0.0" ]
        then
            if [ `echo "${TMPRETSTRING}" | grep "^[[:blank:]]*Status" | grep -c "Unavailable"` -gt 0 ]
            then
                exit_with_msg "Error: The states of some VLUNs are unavailable. Run the \"esxcli upadm show vlun ${cmdtypePara}\" command to check the VLUNs. Repair the VLUNs and try again." "CheckOnlineUpgragePath"
            fi
        else
            if [ `echo "${TMPRETSTRING}" | grep "^[[:blank:]]*Status" | grep -cw "Unavailable"` -gt 0 ]
            then
                exit_with_msg "Error: The states of some VLUNs are unavailable. Run the \"esxcli upadm show vlun ${cmdtypePara}\" command to check the VLUNs. Repair the VLUNs and try again." "CheckOnlineUpgragePath"
            fi
        fi

        vlunStatusNum=`echo "${TMPRETSTRING}" |grep -c "Status"`
        vlunFaultStatusNum=`echo "${TMPRETSTRING}" |grep "Status" |grep -c "Fault"`
        if [[ $vlunStatusNum -eq $vlunFaultStatusNum ]] && [[ $vlunFaultStatusNum -gt 0 ]]
        then
            exit_with_msg "Error: The states of some VLUNs are not correct. Run the \"esxcli upadm check status\" command to check the VLUNs. Repair the VLUNs and try again. " "CheckOnlineUpgragePath"
        fi
    done
    
    Log_info "logical path normal=${normalPathNUM},fault=${faultLogicPathNUM},degraded=${degradedLogicPathNUM}, disable=${disableLogicPathNUM}" "CheckOnlineUpgragePath"
    if [ $faultLogicPathNUM -gt 0 ] || [ $disableLogicPathNUM -gt 0 ] || [ $degradedLogicPathNUM -gt 0 ]
    then
        if [ ${ou_noPathCheck} -ne 1 ]
        then
            exit_with_msg "Error: The states of some paths are not correct. Run the esxcli upadm check status command to check the paths. Repair the paths and try again. " "CheckOnlineUpgragePath"
        else
            let ret=1
        fi
    fi
    
    ExecuteCliRetString "${UPADMDIR}/upadm show path"
    normalPhysicPathNUM=`echo "${TMPRETSTRING}" |grep Normal |awk '{print $(NF-3)}' |grep -c "^Normal$"`
    faultPhysicPathNUM=`echo "${TMPRETSTRING}" |grep Fault |awk '{print $(NF-3)}' |grep -c "^Fault$"`
    disablePhysicPathNUM=`echo "${TMPRETSTRING}" |grep Disable |awk '{print $(NF-3)}' |grep -c "^Disable$"`
    degradedPhysicPathNUM=`echo "${TMPRETSTRING}" |grep degradation |awk '{print $(NF-3)}' |grep -c "^degradation$"`
    
    Log_info "phypath path normal=${normalPhysicPathNUM},fault=${faultPhysicPathNUM},degraded=${degradedPhysicPathNUM}, disable=${disablePhysicPathNUM}" "CheckOnlineUpgragePath"
    if [ $faultPhysicPathNUM -gt 0 ] || [ $disablePhysicPathNUM -gt 0 ] || [ $degradedPhysicPathNUM -gt 0 ]
    then
        if [ ${ou_noPathCheck} -ne 1 ]
        then
            exit_with_msg "Error: The states of some paths are not correct. Run the esxcli upadm check status command to check the paths. Repair the paths and try again. " "CheckOnlineUpgragePath"
        else
            let ret=1
        fi
    fi

    ExecuteCliRetString "${UPADMDIR}/upadm show path"
    checkingPhysicPathNum=`echo "${TMPRETSTRING}" |grep Checking |awk '{print $(NF-2)}' |grep -c "^Checking$"`
    waitingPhysicPathNum=`echo "${TMPRETSTRING}" |grep Waiting |awk '{print $(NF-2)}' |grep -c "^Waiting$"`
    Log_info "phypath path checking=${checkingPhysicPathNum},waiting=${waitingPhysicPathNum}" "CheckOnlineUpgragePath"
    if [  ${checkingPhysicPathNum} -gt 0 ] || [ ${waitingPhysicPathNum} -gt 0 ]
    then
        if [ ${ou_noPathCheck} -ne 1 ]
        then
            exit_with_msg "Error: The states of some paths are not correct. Run the esxcli upadm check status command to check the paths. Repair the paths and try again. " "CheckOnlineUpgragePath"
        else
            let ret=1
        fi
    fi
    
    return ${ret}
}

CheckOnlineUpgrageController()
{
    local ret=0

    ExecuteCliRetString "${UPADMDIR}/upadm check status"
    Log_info "check status" "CheckOnlineUpgrageController"
    COMTROLLER_COUNT=`echo "${TMPRETSTRING}" |grep -c "No redundant controllers"`
    if [ ${COMTROLLER_COUNT} -gt 0 ]
    then
       if [ ${ou_noPathCheck} -ne 1 ]
       then
           exit_with_msg "Error: The states of some paths are not correct. Run the esxcli upadm check status command to check the paths. Repair the paths and try again. " "CheckOnlineUpgrageController"
       else
           let ret=1
       fi
    fi
    
    return ${ret}
}

CheckOnlineUpgrageModule()
{
    local NXUP_EXT=nxup-ext
    local LOAD_BASE_COUNT=0
    local LOAD_EXT_COUNT=0
    
    LOAD_EXT_COUNT=`vmkload_mod -l |grep "${NXUP_EXT}" -c`
    if [ $LOAD_EXT_COUNT -ne 1 ]
    then
        exit_with_msg "Error: The installed UltraPath is not running correctly." "CheckOnlineUpgrageModule"
    fi
    
    LOAD_BASE_COUNT=`vmkload_mod -l |grep "${NXUP_BASE}" -c`
    if [ $LOAD_BASE_COUNT -ne 1 ]
    then
        exit_with_msg "Error: The installed UltraPath is not running correctly." "CheckOnlineUpgrageModule"
    fi
    
    CURENT_LOAD_EXT=`vmkload_mod -l |grep "${NXUP_EXT}" | awk '{print $1}'`
}

CheckOnlineUpgrageUercfgXml()
{
    RestartHostdNote

    CLI_XML_DIFF=`diff /usr/lib/vmware/esxcli/ext/esxcli-ultrapath.xml $BAKDIR/src/esxcli-ultrapath.xml`
    if [ "${CLI_XML_DIFF}" = "" ]
    then
        Log_info "Diff esxcli-ultrapath.xml is same." "CheckOnlineUpgrageUercfgXml"
        RESTART_HOSTD_FLAG=0
    fi
}

CheckOnlineUpgradePrepare()
{
    local checkRet=0

    CheckOnlineUpgrageSystem
    CheckOnlineUpgradeInputParam
    CheckOnlineUpgrageUpadm
    CheckOnlineUpgragePackage
    CheckOnlineUpgragePath
    let checkRet=${checkRet}+$?
    CheckOnlineUpgrageController
    let checkRet=${checkRet}+$?
    
    if [ ${checkRet} -gt 0 ]
    then
        echo -n "The path is abnormal or the Lun is connected to a single controller, which contains potential upgrade risks and cannot meet non-disruptive upgrade conditions. Continue with the non-disruptive upgrade [y]. Stop the non-disruptive upgrade [n]. [y,n,?] : "
        Log_info "The path is abnormal or the Lun is connected to a single controller, which contains potential upgrade risks and cannot meet non-disruptive upgrade conditions. Continue with the non-disruptive upgrade [y]. Stop the non-disruptive upgrade [n]. [y,n,?] : " "CheckOnlineUpgradePrepare"
        while :
        do
            ANSWER=""
            read ANSWER
            if [ "$ANSWER" = "Y" ] || [ "$ANSWER" = "y" ] || [ "$ANSWER" = "YES" ] || [ "$ANSWER" = "Yes" ] || [ "$ANSWER" = "yes" ]
            then
                break
            elif [ "$ANSWER" = "N" ] || [ "$ANSWER" = "n" ] || [ "$ANSWER" = "NO" ] || [ "$ANSWER" = "No" ] || [ "$ANSWER" = "no" ]
            then
                exit_with_msg "The answer is $ANSWER, exit." "CheckOnlineUpgradePrepare"
            else
                TerminaLog_error "        ERROR: Please enter yes or no." "CheckOnlineUpgradePrepare"
                echo -n "The path is abnormal or the Lun is connected to a single controller, which contains potential upgrade risks and cannot meet non-disruptive upgrade conditions. Continue with the non-disruptive upgrade [y]. Stop the non-disruptive upgrade [n]. [y,n,?] : "
                Log_info "The path is abnormal or the Lun is connected to a single controller, which contains potential upgrade risks and cannot meet non-disruptive upgrade conditions. Continue with the non-disruptive upgrade [y]. Stop the non-disruptive upgrade [n]. [y,n,?] : " "CheckOnlineUpgradePrepare"
             fi
        done
    fi
    
    CheckOnlineUpgrageModule
    CheckOnlineUpgrageUercfgXml
}

WaitOnlineUpgradeEsxcli()
{
    WAIT_COUNT=0
    while [ ${WAIT_COUNT} -lt 20 ]
    do
        if [ `esxcli |grep -c "Connection failure"` -ne 1 ]
        then
            break;
        fi

        sleep 2
        WAIT_COUNT=`expr ${WAIT_COUNT} + 1`
    done
}

RestartHostdFun()
{
    local pid
    
    if [ ${RESTART_HOSTD_FLAG} -eq 1 ]
    then
        pid=`ps -c | grep -v grep | grep "hbr" | awk '{print $1}'`
        [ "${pid}" != "" ] && kill -9 ${pid}
        Log_info "Excuting hostd restart." "RestartHostdFun"
        /etc/init.d/hostd restart >> "${LOGFILE}" 2>&1
        [ $? -ne 0 ] && return 1
        WaitOnlineUpgradeEsxcli
    fi
}

OnlineUpgradeInstallNewPackage()
{
    Log_info "Excuting esxcli software vib install -d ${PARAMETER}/${PKGNAME}" "OnlineUpgradeInstallNewPackage"
    if [ $CURPKG_IS_CERT_FLAG -eq 0 ]
    then
        esxcli software vib install -d ${PARAMETER}/${PKGNAME} --no-sig-check > $BAKDIR/$INSTALLRST
    else
        esxcli software vib install -d ${PARAMETER}/${PKGNAME} > $BAKDIR/$INSTALLRST
    fi
	
    if [ $? -ne 0 ]
    then
        exit_with_msg "Alarm: The online upgrade of UltraPath to ${NEW_VERSION} failed. UltraPath was successfully rolled back to ${CURRENT_VERSION}. Contact the technical support." "OnlineUpgradeInstallNewPackage"
    fi
    
    if [ -f $BAKDIR/$INSTALLRST ]
    then
        ISINSTALLSAME=`cat $BAKDIR/$INSTALLRST | grep -c "Host is not changed"`
        if [ $ISINSTALLSAME -eq 1 ]
        then
            Log_info "Excuting install the same version package." "OnlineUpgradeInstallNewPackage"
        fi
    fi
}

OnlineUpgradeActivateNewDrive()
{
    local LOAD_EXT_A_COUNT=0
    local OUC_RET=0
    
    if [ $BAKPKG_FULL_FLAG -eq 0 ]
    then
        if [ -d ${BAKDIR}/${OLD_PKG_UNZIP}/src ]
        then
            rm -rf ${BAKDIR}/${OLD_PKG_UNZIP}/src/*
        else
            mkdir ${BAKDIR}/${OLD_PKG_UNZIP}/src
        fi
        CopyFileFun /usr/lib/cim/libultrapathprovider.so ${BAKDIR}/${OLD_PKG_UNZIP}/src/
        CopyFileFun /usr/lib/vmware/vmkmod/mp-plugin-ultrapath ${BAKDIR}/${OLD_PKG_UNZIP}/src/
        CopyFileFun /usr/lib/vmware/vmkmod/nxup-ext-a ${BAKDIR}/${OLD_PKG_UNZIP}/src/
        CopyFileFun /usr/lib/vmware/vmkmod/nxup-ext-b ${BAKDIR}/${OLD_PKG_UNZIP}/src/
        CopyFileFun /opt/UltraPath/bin/upadm ${BAKDIR}/${OLD_PKG_UNZIP}/src/
        CopyFileFun /usr/lib/vmware/esxcli/ext/esxcli-ultrapath.xml ${BAKDIR}/${OLD_PKG_UNZIP}/src/
    fi

    CopyFileFun ${BAKDIR}/src/libultrapathprovider.so /usr/lib/cim/
    CopyFileFun ${BAKDIR}/src/mp-plugin-ultrapath /usr/lib/vmware/vmkmod/
    CopyFileFun ${BAKDIR}/src/nxup-ext-a /usr/lib/vmware/vmkmod/
    CopyFileFun ${BAKDIR}/src/nxup-ext-b /usr/lib/vmware/vmkmod/
    CopyFileFun ${BAKDIR}/src/upadm /opt/UltraPath/bin/
    CopyFileFun ${BAKDIR}/src/esxcli-ultrapath.xml /usr/lib/vmware/esxcli/ext/
    chmodExec 500 /opt/UltraPath/bin/upadm
    chmodExec 500 /usr/lib/cim/libultrapathprovider.so
    if [ "$OS_VERSION" == "5.0.0" ]
    then
        CopyFileFun ${BAKDIR}/src/upadm /opt/UltraPath/bin/upadmpref
        chmodExec 500 /opt/UltraPath/bin/upadmpref
    fi
    
    CopyFileFun $BAKDIR/$UPCONFIGOLD /etc/cim/config/
    CopyFileFun $BAKDIR/$UPCONFIG /etc/cim/config/
    
    chmodExec 700 $BAKDIR/cfg_upgrade

    Log_info "Excute $BAKDIR/cfg_upgrade -r8." "OnlineUpgradeActivateNewDrive"
    $BAKDIR/cfg_upgrade -r8
    if [ $? -ne 0 ]
    then
        Log_error "Excute $BAKDIR/cfg_upgrade -r8 failed." "OnlineUpgradeActivateNewDrive"
    fi
   
    Log_info "Excuting sfcbd-watchdog restart." "OnlineUpgradeActivateNewDrive"
    /etc/init.d/sfcbd-watchdog restart >> "${LOGFILE}" 2>&1

    chmodExec 500 ${BAKDIR}/${OUCFILE}
    touch /etc/cim/log/ouc_log.txt
    
    if [ "$CURENT_LOAD_EXT" == "$NXUP_EXT_B" ]
    then
        LOAD_EXT_A_COUNT=`vmkload_mod -l |grep $NXUP_EXT_A -c`
        if [ ${LOAD_EXT_A_COUNT} -gt 0 ]
        then
            Log_info "Excuting vmkload_mod -u $NXUP_EXT_A" "OnlineUpgradeActivateNewDrive"
            vmkload_mod -u ${NXUP_EXT_A} >> "${LOGFILE}" 2>&1
            if [ $? -eq 0 ]
            then
                LOAD_EXT_A_COUNT=0
            else
                Log_error "Excute unload new nxup ext module fail." "OnlineUpgradeActivateNewDrive"
            fi
        fi
    fi

    if [ ${LOAD_EXT_A_COUNT} -eq 0 ] || [ "${CURENT_LOAD_EXT}" == "${NXUP_EXT_A}" ]
    then
        Log_info "Excuting ouc." "OnlineUpgradeActivateNewDrive"
        ${BAKDIR}/${OUCFILE}
        OUC_RET=$?
        if [ ${OUC_RET} -eq 0 ]
        then
            RestartHostdFun
            Log_info "Excuting sfcbd-watchdog restart." "OnlineUpgradeActivateNewDrive"
            /etc/init.d/sfcbd-watchdog restart >> "${LOGFILE}" 2>&1

            rm -f ${BACKUP_PKG_DIR}/${OLD_PKG_NAME}
            CopyFileFun ${PARAMETER}/${PKGNAME} ${BACKUP_PKG_DIR}/
            return 0
        else
            if [ ${OUC_RET} -eq 20 ]
            then
                RestartHostdFun
                Log_info "Excuting sfcbd-watchdog restart." "OnlineUpgradeActivateNewDrive"
                /etc/init.d/sfcbd-watchdog restart >> "${LOGFILE}" 2>&1

                rm -f ${BACKUP_PKG_DIR}/${OLD_PKG_NAME}
                CopyFileFun ${PARAMETER}/${PKGNAME} ${BACKUP_PKG_DIR}/
                return 20
            fi
        fi
        Log_error "Excute ouc fail, error message look for ouc_log file." "OnlineUpgradeActivateNewDrive"
    fi

    return 1
}

OnlineUpgradeRollback()
{
    if [ $BAKPKG_FULL_FLAG -eq 1 ]
    then
        Log_info "Excuting esxcli software vib remove -n ultrapath.cli" "OnlineUpgradeRollback"
        esxcli software vib remove -n ultrapath.cli >> "${LOGFILE}" 2>&1
        if [ $? -ne 0 ]
        then
            exit_with_msg "Error: Online upgrade fail, rollback fail, contact the technical service." "OnlineUpgradeRollback"
        fi

        Log_info "Excuting esxcli software vib remove -n ultrapath.cim" "OnlineUpgradeRollback"
        esxcli software vib remove -n ultrapath.cim >> "${LOGFILE}" 2>&1
        if [ $? -ne 0 ]
        then
            exit_with_msg "Error: Online upgrade fail, rollback fail, contact the technical service." "OnlineUpgradeRollback"
        fi

        Log_info "Excuting esxcli software vib remove -n ultrapath.mpp" "OnlineUpgradeRollback"
        esxcli software vib remove -n ultrapath.mpp >> "${LOGFILE}" 2>&1
        if [ $? -ne 0 ]
        then
            exit_with_msg "Error: Online upgrade fail, rollback fail, contact the technical service." "OnlineUpgradeRollback"
        fi

        Log_info "Excuting esxcli software vib install -d ${BAKDIR}/${OLD_PKG_NAME}" "OnlineUpgradeRollback"
        if [ $CURPKG_IS_CERT_FLAG -eq 0 ]
        then
            esxcli software vib install -d ${BAKDIR}/${OLD_PKG_NAME} --no-sig-check >> "${LOGFILE}" 2>&1
        else
            esxcli software vib install -d ${BAKDIR}/${OLD_PKG_NAME} >> "${LOGFILE}" 2>&1
        fi
		
        if [ $? -ne 0 ]
        then
            exit_with_msg "Alarm: The online upgrade of UltraPath to ${NEW_VERSION} failed. Rollback of UltraPath to ${CURRENT_VERSION} failed. Contact the technical support." "OnlineUpgradeRollback"
        fi

        CopyFileFun ${BAKDIR}/${OLD_PKG_NAME} ${BACKUP_PKG_DIR}/${OLD_PKG_NAME}
    fi

    CopyFileFun ${BAKDIR}/${OLD_PKG_UNZIP}/src/libultrapathprovider.so /usr/lib/cim/
    CopyFileFun ${BAKDIR}/${OLD_PKG_UNZIP}/src/mp-plugin-ultrapath /usr/lib/vmware/vmkmod/
    CopyFileFun ${BAKDIR}/${OLD_PKG_UNZIP}/src/nxup-ext-a /usr/lib/vmware/vmkmod/
    CopyFileFun ${BAKDIR}/${OLD_PKG_UNZIP}/src/nxup-ext-b /usr/lib/vmware/vmkmod/
    CopyFileFun ${BAKDIR}/${OLD_PKG_UNZIP}/src/upadm /opt/UltraPath/bin/
    CopyFileFun ${BAKDIR}/${OLD_PKG_UNZIP}/src/esxcli-ultrapath.xml /usr/lib/vmware/esxcli/ext/
    chmodExec 500 /opt/UltraPath/bin/upadm
    chmodExec 500 /usr/lib/cim/libultrapathprovider.so
    
    if [ "$OS_VERSION" == "5.0.0" ]
    then
        CopyFileFun ${BAKDIR}/${OLD_PKG_UNZIP}/src/upadm /opt/UltraPath/bin/upadmpref
        chmodExec 500 /opt/UltraPath/bin/upadmpref
    fi
    
    if [ -f ${CONDIR}/${UPCONFIGOLD} ]
    then
        mv ${CONDIR}/${UPCONFIGOLD} ${CONDIR}/${UPCONFIG}
    else
        CopyFileFun $BAKDIR/$UPCONFIGOLD ${CONDIR}
        mv ${CONDIR}/${UPCONFIGOLD} ${CONDIR}/${UPCONFIG}
    fi  

    RestartHostdFun
    Log_info "Excuting sfcbd-watchdog restart." "OnlineUpgradeRollback"
    /etc/init.d/sfcbd-watchdog restart >> "${LOGFILE}" 2>&1

    if [ $BAKPKG_FULL_FLAG -eq 1 ]
    then
        exit_with_msg "Alarm: The online upgrade of UltraPath to ${NEW_VERSION} failed. UltraPath was successfully rolled back to ${CURRENT_VERSION}. Contact the technical support." "OnlineUpgradeRollback"
    else
        exit_with_msg "Alarm: The online upgrade of UltraPath to ${NEW_VERSION} failed. You can try to perform online upgrade again, or restart the host to make the new version take effect." "OnlineUpgradeRollback"
    fi
}

InstallOnlineUpgrade()
{  
    local ACTIVATE_RET=0
    
    TerminaLog_info "Start the online upgrade." "InstallOnlineUpgrade"

    TerminaLog_info "Step 1: Perform the pre-upgrade check."  "InstallOnlineUpgrade"
    CheckOnlineUpgradePrepare

    trap '' INT QUIT HUP TERM
    
    CopyFileFun ${CONDIR}/user_cfg.xml ${CONDIR}/${UPCONFIGOLD}
    mv ${CONDIR}/${UPCONFIGOLD} ${BAKDIR}/${UPCONFIGOLD}

    TerminaLog_info "Step 2: Update install UltraPath ${NEW_VERSION}." "InstallOnlineUpgrade"
    OnlineUpgradeInstallNewPackage

    TerminaLog_info "Step 3: Activate UltraPath ${NEW_VERSION} online." "InstallOnlineUpgrade"
    OnlineUpgradeActivateNewDrive
    ACTIVATE_RET=$?
    if [ ${ACTIVATE_RET} -ne 0 ]
    then
        if [ ${ACTIVATE_RET} -eq 20 ]
        then
            exit_with_msg "Alarm: UltraPath ${NEW_VERSION} was activated successfully online. There are residues of UltraPath ${CURRENT_VERSION}. Contact the technical support." "InstallOnlineUpgrade"
        else
            TerminaLog_error "Error: Online activation of UltraPath ${NEW_VERSION} failed. A rollback started." "InstallOnlineUpgrade"
            OnlineUpgradeRollback
        fi
    fi
    CheckOnlineUpgrageSpecificationAfterOU
    TerminaLog_info "The online upgrade is complete." "InstallOnlineUpgrade"
    TerminaLog_info "Note: The output of the esxcli software vib list command shows that the UltraPath version is the pre-upgrade one. Run the esxcli upadm show version command to check the version of the current UltraPath." "InstallOnlineUpgrade"
}

CheckOem()
{
    if [ ! -d Config ]
    then
        i10s_isOEM=0
    elif [ -d Config ] && [ ! -f Config/up_vendor.conf ] && [ ! -f Config/upImport.sh ]
    then
        i10s_isOEM=0
    else
        i10s_isOEM=1
    fi
    
    if [ ${i10s_isOEM} -eq 1 ]
    then
        if [ ! -f Config/up_vendor.conf ] || [ ! -f Config/upImport.sh ]
        then
            exit_with_msg "Config directory is not valid, exit install." "CheckOem"
        fi

        cd ./Config
        sh upImport.sh -c
        if [ $? -ne 0 ]
        then
            exit_with_msg "Check config file fail, exit install." "CheckOem"
        fi
        cd ..
    fi
}

ImportOem()
{
    if [ ${i10s_isOEM} -eq 1 ]
    then
        if [ ! -f Config/up_vendor.conf ] || [ ! -f Config/upImport.sh ]
        then
            TerminaLog_error "ERROR: The up_vendor.conf file fails to be imported to the system because some files in the Config folder are missing."
        else
            cd ./Config
            sh upImport.sh
            cd ..
        fi
    fi
}

CheckOfflineUpgrageUercfgXml()
{
    CLI_XML_DIFF=`diff /usr/lib/vmware/esxcli/ext/esxcli-ultrapath.xml $BAKDIR/src/esxcli-ultrapath.xml`
    if [ "${CLI_XML_DIFF}" = "" ]
    then
        Log_info "Diff esxcli-ultrapath.xml is same." "CheckOnlineUpgrageUercfgXml"
        RESTART_HOSTD_FLAG=0
    else
        RESTART_HOSTD_FLAG=1
        RestartHostdNote
    fi
}

SaveVLunInfo()
{
    local invalidInputNum
    local typePara
    
    ExecuteCliRetString "${UPADMDIR}/upadm show vlun type=all"
    invalidInputNum=`echo "${TMPRETSTRING}" |grep -c "Invalid input"`
    if [ $invalidInputNum -gt 0 ]
    then
        typePara=""
    else
        typePara="type=all"
    fi

    ExecuteCliRetString "${UPADMDIR}/upadm show vlun ${typePara}"
    
    echo "${TMPRETSTRING}" | grep naa | awk '{print $2}' | uniq > /tmp/.vlunname
}

OfflineUpgradeCheckVLunInfo()
{
    if [ ! -f /tmp/.vlunname ]
    then
        Log_info "Vlun info file does not exist." "OfflineUpgradeCheckVLunInfo"
        return 1
    fi
    
    cat /tmp/.vlunname |while read line  
    do  
        VLUNNAME_NUM=`esxcli storage core device list |grep -c $line`
        if [ $VLUNNAME_NUM -gt 0 ]
        then
            VLUNNAME_PLUGIN=`esxcli storage core device list |grep -A6 $line |grep -c "\<MP_PLUGIN_ULTRAPATH\>"`
            if [ ${VLUNNAME_PLUGIN} -eq 0 ]
            then
                Log_info "Reclaim ${line}." "OfflineUpgradeCheckVLunInfo"
                esxcli storage core claiming reclaim -d ${line} >> "${LOGFILE}" 2>&1
                if [ $? -ne 0 ]
                then
                    Log_info "Reclaim ${line} failed." "OfflineUpgradeCheckVLunInfo"
                    return 1
                fi
            fi
        fi
    done 
    if [ $? -ne 0 ]
    then
        return 1
    fi

    return 0
}

OfflineUpgradeUnClaim()
{
    Log_info "Excuting esxcli storage core claiming unclaim -t plugin -P MP_PLUGIN_ULTRAPATH." "OfflineUpgradeUnClaim"
    esxcli storage core claiming unclaim -t plugin -P MP_PLUGIN_ULTRAPATH >> "${LOGFILE}" 2>&1
    if [ $? -ne 0 ]
    then
        TerminaLog_error "Failed to remove the device. Please restart the host to make the new version take effect after the upgrade is complete." "OfflineUpgradeUnClaim"
        return 1
    fi
}

OfflineUpgradeUnloadMod()
{
    local LOAD_EXT_A_COUNT=0
    local LOAD_EXT_B_COUNT=0
    local LOAD_BASE_COUNT=0
    local retryTimes=0
    local ret=0

    Log_info "Excuting sfcbd-watchdog stop." "OfflineUpgradeUnloadMod"
    while [ ${retryTimes} -lt 3 ]
    do
        /etc/init.d/sfcbd-watchdog stop >> "${LOGFILE}" 2>&1
        ret=$?
        [ $ret -eq 0 ] && break
        let retryTimes=retryTimes+1
        Log_info "Excuting sfcbd-watchdog stop, retry ${retryTimes} times." "OfflineUpgradeUnloadMod"
    done
    
    if [ $ret -ne 0 ] 
    then
        TerminaLog_error "Failed to stop the sfcdb-watchdog service. Please restart the host to make the new version take effect after the upgrade is complete." "OfflineUpgradeUnloadMod"
        return 1
    fi
    
    OfflineUpgradeUnClaim
    if [ $? -ne 0 ]
    then
        /etc/init.d/sfcbd-watchdog start >> "${LOGFILE}" 2>&1
        esxcfg-rescan -A >> "${LOGFILE}" 2>&1
        return 1
    fi
    
    LOAD_EXT_A_COUNT=`vmkload_mod -l |grep $NXUP_EXT_A -c`
    if [ ${LOAD_EXT_A_COUNT} -gt 0 ]
    then
        Log_info "Excuting vmkload_mod -u $NXUP_EXT_A" "OfflineUpgradeUnloadMod"
        vmkload_mod -u ${NXUP_EXT_A} >> "${LOGFILE}" 2>&1
        if [ $? -ne 0 ]
        then
            TerminaLog_error "Failed to unload the old driver. Please restart the host to make the new version take effect after the upgrade is complete." "OfflineUpgradeUnloadMod"
            /etc/init.d/sfcbd-watchdog start >> "${LOGFILE}" 2>&1
            esxcfg-rescan -A >> "${LOGFILE}" 2>&1
            return 1
        fi
    fi
    
    LOAD_EXT_B_COUNT=`vmkload_mod -l |grep $NXUP_EXT_B -c`
    if [ ${LOAD_EXT_B_COUNT} -gt 0 ]
    then
        Log_info "Excuting vmkload_mod -u $NXUP_EXT_B" "OfflineUpgradeUnloadMod"
        vmkload_mod -u ${NXUP_EXT_B} >> "${LOGFILE}" 2>&1
        if [ $? -ne 0 ]
        then
            TerminaLog_error "Failed to unload the old driver, please restart the host to make the new version take effect after the upgrade is complete." "OfflineUpgradeUnloadMod"
            /etc/init.d/sfcbd-watchdog start >> "${LOGFILE}" 2>&1
            esxcfg-rescan -A >> "${LOGFILE}" 2>&1
            return 1
        fi
    fi
    
    LOAD_BASE_COUNT=`vmkload_mod -l |grep "${NXUP_BASE}" -c`
    if [ ${LOAD_BASE_COUNT} -gt 0 ]
    then
        Log_info "Excuting vmkload_mod -u $NXUP_BASE" "OfflineUpgradeUnloadMod"
        vmkload_mod -u ${NXUP_BASE} >> "${LOGFILE}" 2>&1
        if [ $? -ne 0 ]
        then
            TerminaLog_error "Failed to unload the old driver, please restart the host to make the new version take effect after the upgrade is complete." "OfflineUpgradeUnloadMod"
            Log_info "Excuting vmkload_mod $NXUP_EXT_A" "OfflineUpgradeUnloadMod"
            vmkload_mod $NXUP_EXT_A >> "${LOGFILE}" 2>&1
            if [ $? -ne 0 ]
            then
                /etc/init.d/sfcbd-watchdog start >> "${LOGFILE}" 2>&1
                esxcfg-rescan -A >> "${LOGFILE}" 2>&1
            else
                Log_info "Roll back successfully, check vlun info" "OfflineUpgradeUnloadMod"
                OfflineUpgradeCheckVLunInfo
                esxcfg-rescan -A >> "${LOGFILE}" 2>&1
            fi
            return 1
        fi
    fi
    
    return 0
}

OfflineUpgradeRollBackLoadMod()
{
    Log_info "Roll back modules, modules failed at $1" "OfflineUpgradeRollBackLoadMod"
    local ret=0
    if [ "$1" = "" ] && [ "$1" != "base" ] && [ "$1" != "ext" ] 
    then
        return 0
    fi
    if [ "$1" = "ext" ]
    then
        vmkload_mod -u ${NXUP_BASE} >> "${LOGFILE}" 2>&1
        ret=$?
    fi
    if [ $ret -eq 0 ]
    then
        vmkload_mod ${NXUP_BASE} >> "${LOGFILE}" 2>&1
        ret=$?
    fi
    if [ $ret -eq 0 ]
    then
        vmkload_mod ${NXUP_EXT_A} >> "${LOGFILE}" 2>&1
        ret=$?
    fi
    if [ $ret -eq 0 ]
    then
        Log_info "Roll back successfully, check vlun info." "OfflineUpgradeRollBackLoadMod"
        OfflineUpgradeCheckVLunInfo
    fi
    esxcfg-rescan -A >> "${LOGFILE}" 2>&1
    return $ret
}

OfflineUpgradeCheckPeLun()
{
    local ret
    local invalidInputNum
    
    ExecuteCliRetString "${UPADMDIR}/upadm show vlun type=pe"
    invalidInputNum=`echo "${TMPRETSTRING}" |grep -c "Invalid input"`
    if [ $invalidInputNum -gt 0 ]
    then
        return 0
    fi

    ret=`echo "${TMPRETSTRING}" | grep -c "find any vluns"`
    if [ $ret -ne 0 ]
    then
        return 0
    fi
    return 1
}

OfflineUpgradeCheckModules()
{
    local NXUP_EXT="nxup-ext"
    local LOAD_EXT_COUNT=0
    
    LOAD_EXT_COUNT=`vmkload_mod -l |grep "${NXUP_EXT}" -c`
    
    if [ $LOAD_EXT_COUNT -gt 1 ]
    then
        return 1
    fi
}

OfflineUpgradeReplaceNewFiles()
{
    CopyFileFun ${BAKDIR}/src/libultrapathprovider.so /usr/lib/cim/
    CopyFileFun ${BAKDIR}/src/mp-plugin-ultrapath /usr/lib/vmware/vmkmod/
    CopyFileFun ${BAKDIR}/src/nxup-ext-a /usr/lib/vmware/vmkmod/
    CopyFileFun ${BAKDIR}/src/nxup-ext-b /usr/lib/vmware/vmkmod/
    CopyFileFun ${BAKDIR}/src/upadm /opt/UltraPath/bin/
    CopyFileFun ${BAKDIR}/src/esxcli-ultrapath.xml /usr/lib/vmware/esxcli/ext/
    chmodExec 500 /opt/UltraPath/bin/upadm
    chmodExec 500 /usr/lib/cim/libultrapathprovider.so
    if [ "$OS_VERSION" == "5.0.0" ]
    then
        CopyFileFun ${BAKDIR}/src/upadm /opt/UltraPath/bin/upadmpref
        chmodExec 500 /opt/UltraPath/bin/upadmpref
    fi
    
    Log_info "Replace files successfully." "OfflineUpgradeReplaceNewFiles"
}

OfflineUpgradeLoadMod()
{
    local LOAD_EXT_A_COUNT=0
    local LOAD_EXT_B_COUNT=0
    local LOAD_BASE_COUNT=0
    
    LOAD_BASE_COUNT=`vmkload_mod -l |grep ${NXUP_BASE} -c`
    if [ ${LOAD_BASE_COUNT} -ne 0 ]
    then
        Log_info "${NXUP_BASE} exists when load new modules, unload it." "OfflineUpgradeLoadMod"
        vmkload_mod -u ${NXUP_BASE} >> "${LOGFILE}" 2>&1
        if [ $? -ne 0 ]
        then
            TerminaLog_error "Failed to load the new driver. Please restart the host to make the new version take effect." "OfflineUpgradeLoadMod"
            esxcfg-rescan -A >> "${LOGFILE}" 2>&1
            return 1
        fi
    fi
    vmkload_mod ${BAKDIR}/src/mp-plugin-ultrapath >> "${LOGFILE}" 2>&1
    if [ $? -ne 0 ]
    then
        TerminaLog_error "Failed to load the new driver, please restart the host to make the new version take effect." "OfflineUpgradeLoadMod"
        OfflineUpgradeRollBackLoadMod "base"
        return 1
    fi
    vmkload_mod ${BAKDIR}/src/nxup-ext-a >> "${LOGFILE}" 2>&1
    if [ $? -ne 0 ]
    then
        TerminaLog_error "Failed to load the new driver, please restart the host to make the new version take effect." "OfflineUpgradeLoadMod"
        OfflineUpgradeRollBackLoadMod "ext"
        return 1
    fi
    Log_info "Check vlun info." "OfflineUpgradeLoadMod"
    OfflineUpgradeCheckVLunInfo
    if [ $? -ne 0 ]
    then
        Log_info "Modules load successfully, but check vlun info failed." "OfflineUpgradeLoadMod"
        TerminaLog_error "Failed to load the new driver, please restart the host to make the new version take effect." "OfflineUpgradeLoadMod"
        return 1
    fi
    
    OfflineUpgradeReplaceNewFiles
    
    return 0
}

OfflineUpgradeActivateNewDrive()
{
    local ret=""
    OfflineUpgradeLoadMod
    if [ $? -ne 0 ]
    then
        Log_info "Activate new drive fail." "OfflineUpgradeActivateNewDrive"
        /etc/init.d/sfcbd-watchdog start >> "${LOGFILE}" 2>&1
        return 1
    fi
    
    Log_info "Excuting sfcbd-watchdog start." "OfflineUpgradeLoadMod"
    /etc/init.d/sfcbd-watchdog start >> "${LOGFILE}" 2>&1
    if [ $? -ne 0 ]
    then
        TerminaLog_error "Failed to start the sfcdb-watchdog service, please restart the host to make the new version take effect. " "OfflineUpgradeLoadMod"
        return 1
    fi
    
    if [ ${RESTART_HOSTD_FLAG} -eq 1 ]
    then
        Log_info "Excuting hostd restart." "OfflineUpgradeActivateNewDrive"
        RestartHostdFun
        if [ $? -ne 0 ]
        then
            TerminaLog_error "Failed to start the hostd service, please restart the host to make the new version take effect." "OfflineUpgradeActivateNewDrive"
            return 1
        fi
    fi
    TerminaLog_info "Host is rescanning. Please wait......" "OfflineUpgradeActivateNewDrive"
    esxcfg-rescan -A >> "${LOGFILE}" 2>&1
    return 0
}

OfflineUpgradeNotReboot()
{
    CheckIfSanBoot
    if [ $? -eq 1 ]
    then
        Log_info "The system is boot from san." "OfflineUpgradeNotReboot"
        NEEDBOOTFLAG=1;
        return 1
    fi

    OfflineUpgradeCheckModules
    if [ $? -eq 1 ]
    then
        Log_info "There are more than 1 ext modules." "OfflineUpgradeNotReboot"
        NEEDBOOTFLAG=1;
        return 1
    fi
    
    OfflineUpgradeCheckPeLun
    if [ $? -eq 1 ]
    then
        Log_info "Some PE LUNs are managed by UltraPath. Please restart the host to make the new version take effect." "OfflineUpgradeNotReboot"
        NEEDBOOTFLAG=1;
        return 1
    fi

    OfflineUpgradeUnloadMod
    if [ $? -eq 1 ]
    then
        Log_error "Failed to unload mod." "OfflineUpgradeNotReboot"
        NEEDBOOTFLAG=1;
        return 1
    fi
    
    OfflineUpgradeActivateNewDrive
    if [ $? -ne 0 ]
    then
        NEEDBOOTFLAG=1;
    fi
}

CheckUnattendParameter()
{
    if [ $# -gt 2 ]
    then
         exit_with_msg "Error: The parameter value is incorrect." "CheckUnattendParameter"
         exit 1
    fi
    
    if [ $# -eq 1 ]
    then
        exit_with_msg "Error: The configuration file unattend_install.conf is not specified after the -f parameter." "CheckUnattendParameter"
        exit 1
    fi
    
    Unattend_Conf_File="$2"
    
    Unattend_File_Name=`echo "${Unattend_Conf_File##*/}"|sed 's/^[ \t]*//g'|sed 's/[ \t]*$//g'`
    if [ "${Unattend_File_Name}" != "unattend_install.conf" ]
    then
        exit_with_msg "Error: The name of the specified configuration file is not unattend_install.conf." "CheckUnattendParameter"
        exit 1        
    fi
    
    if [ ! -f ${Unattend_Conf_File} ]
    then
        exit_with_msg  "Error: The specified configuration file unattend_install.conf cannot be found."  "CheckUnattendParameter"
        exit 1
    fi
}

UltraPathRestart=""
UltraPathLunConfContinue=""
GetUattendConfFile()
{
    if [ ! -f ${Unattend_Conf_File} ]
    then
        exit_with_msg "Error: The specified configuration file unattend_install.conf cannot be found." "GetUattendConfFile"
        exit 1
    fi
    
    sed -i 's/\r$//g' ${Unattend_Conf_File}
    
    lastLine=""
    
    lastLine=`tail -n 1 ${Unattend_Conf_File}`
    if [ ! -z "${lastLine}" ]
    then
        echo "" >> "${Unattend_Conf_File}"
    fi
 
    
    UltraPathRestartFlag=0
    LunExistContinueFlag=0
    while read line
    do
        unattend_valid_line=`echo "$line" | grep -Ev "^$|^#"`
        if [ "${unattend_valid_line}" = "" ]
        then
            continue
        fi
        
        _UnattendPara=`echo $line | cut -d'=' -f1 |sed s/^[[:space:]]//g|sed s/[[:space:]]$//g`
        _UnattendParaVal=`echo $line | cut -d'=' -f2|sed s/^[[:space:]]//g|sed s/[[:space:]]$//g`
        
        if [ "${_UnattendPara}" = "restart" ] && [ ${UltraPathRestartFlag} -eq 1 ]
        then
            exit_with_msg "Error: The category or number of configuration items is incorrect." "GetUattendConfFile"
            exit 1
        fi

        if [ "${_UnattendPara}" = "lun_exist_continue" ] && [ ${LunExistContinueFlag} -eq 1 ]
        then
            exit_with_msg "Error: The category or number of configuration items is incorrect." "GetUattendConfFile"
            exit 1
        fi
        
        
        if [ "${_UnattendPara}" = "restart" ] 
        then
            if [ "$_UnattendParaVal" = "Y" ] || [ "$_UnattendParaVal" = "y" ] || [ "$_UnattendParaVal" = "N" ] || [ "$_UnattendParaVal" = "n" ] 
            then
                UltraPathRestart=$_UnattendParaVal
            else
                exit_with_msg "Error: The value of a configuration item is incorrect. " "GetUattendConfFile"
                exit 1
            fi
            UltraPathRestartFlag=1
        elif [ "${_UnattendPara}" = "lun_exist_continue" ]
        then
            if [ "$_UnattendParaVal" = "Y" ] || [ "$_UnattendParaVal" = "y" ] || [ "$_UnattendParaVal" = "N" ] || [ "$_UnattendParaVal" = "n" ]  
            then
                UltraPathLunConfContinue=$_UnattendParaVal
            else
                exit_with_msg "Error: The value of a configuration item is incorrect. " "GetUattendConfFile"
                exit 1
            fi
            LunExistContinueFlag=1
        else
            exit_with_msg "Error: The category or number of configuration items is incorrect. " "GetUattendConfFile"
            exit 1
        fi
        
    done < ${Unattend_Conf_File}
    
    if [ "${UltraPathRestart}" = "" ]
    then
        exit_with_msg "Error: The category or number of configuration items is incorrect. " "GetUattendConfFile"
        exit 1
    fi
    
    if [ "${UltraPathLunConfContinue}" = "" ]
    then
        print_error "1043" "The category or number of configuration items is incorrect." "GetUattendConfFile"
        exit 1
    fi
}

CheckPathSpace()
{
    if [ `pwd | grep -c " "` -ne 0 ]
    then
        exit_with_msg "Error: there is space key in the directory[`pwd`]." "CheckPathSpace"
    fi
}

SetNewVersionAndOUSupport()
{
    if [ $CURPKG_IS_CERT_FLAG -eq 0 ]
    then
        NEW_VERSION="21.02.046"
        if [ "${PKG_OSNAME}" = "ESXi6.x" ]
        then
            OU_SUPPORT_VER=",8.05.001,8.05.002,8.05.003,8.05.004,8.05.005,8.05.006,8.05.007,8.05.008,8.06.001,8.06.002,8.06.003,8.06.004,8.06.005,8.06.006,8.06.007,8.06.010,8.06.020,8.06.021,8.06.023,8.06.025,8.06.026,8.06.027,8.06.028,8.06.029,8.06.030,8.06.031,8.06.032,8.06.033,8.06.040,8.06.041,8.06.042,8.06.043,8.06.044,8.06.045,8.06.047,8.06.048,8.06.049,8.06.050,8.06.051,8.06.052,8.06.053,8.06.060,8.06.061,21.01.003,21.01.004,21.01.005,21.01.006,21.01.010,21.01.011,21.01.012,21.01.013,21.01.015,21.01.016,21.01.017,21.01.019,21.01.020,21.01.021,21.01.022,21.01.023,21.01.024,21.02.002,21.02.003,21.02.005,21.02.006,21.02.007,21.02.010,21.02.011,21.02.012,21.02.013,21.02.015,21.02.016,21.02.018,21.02.019,21.02.020,21.02.021,21.02.022,21.02.023,21.02.024,21.02.025,21.02.026,21.02.027,21.01.026,21.02.028,21.02.029,21.02.030,21.02.031,21.02.032,21.02.033,21.02.034,21.02.038,21.02.039,21.02.040,21.02.041,21.02.042,21.02.043,21.02.044,21.02.045,21.02.046"
        else
            OU_SUPPORT_VER=",8.03.021,8.03.022,8.03.023,8.03.024,8.03.025,8.03.026,8.03.028,8.03.032,8.05.001,8.05.002,8.05.003,8.05.004,8.05.005,8.05.006,8.05.007,8.05.008,8.06.001,8.06.002,8.06.003,8.06.004,8.06.005,8.06.006,8.06.007,8.06.010,8.06.020,8.06.021,8.06.023,8.06.025,8.06.026,8.06.027,8.06.028,8.06.029,8.06.030,8.06.031,8.06.032,8.06.033,8.06.040,8.06.041,8.06.042,8.06.043,8.06.044,8.06.045,8.06.047,8.06.048,8.06.049,8.06.050,8.06.051,8.06.052,8.06.053,8.06.060,8.06.061,21.01.003,21.01.004,21.01.005,21.01.006,21.01.010,21.01.011,21.01.012,21.01.013,21.01.015,21.01.016,21.01.017,21.01.019,21.01.020,21.01.021,21.01.022,21.01.023,21.01.024,21.02.002,21.02.003,21.02.005,21.02.006,21.02.007,21.02.010,21.02.011,21.02.012,21.02.013,21.02.015,21.02.016,21.02.018,21.02.019,21.02.020,21.02.021,21.02.022,21.02.023,21.02.024,21.02.025,21.02.026,21.02.027,21.01.026,21.02.028,21.02.029,21.02.030,21.02.031,21.02.032,21.02.033,21.02.034,21.02.038,21.02.039,21.02.040,21.02.041,21.02.042,21.02.043,21.02.044,21.02.045,21.02.046"
        fi
    else
        NEW_VERSION="21.02.046"
        if [ "${PKG_OSNAME}" = "ESXi6.x" ]
        then
            OU_SUPPORT_VER=",8.05.007,8.06.010,8.06.045,8.06.061,21.01.006,21.01.015,21.01.024,21.02.023,21.02.034,21.02.046"
        else
            OU_SUPPORT_VER=",8.03.021,8.03.026,8.03.028,8.05.007,8.06.010,8.06.045,8.06.061,21.01.006,21.01.015,21.01.024,21.02.023,21.02.034,21.02.046"
        fi
    fi
	
    return 0
}

CheckUser
CheckPathSpace
CheckOverlapUpgrade
WarningNotSupportCertified
GetParameter
CheckAttendInstallSystemInfo()
{
    CheckOem       
    CheckOSType
    CheckOSVersion
    CheckSystem
    CheckPackageExist
    CheckPackageNum
    CreateBakDir
    CreateBackupPackageDir
    UnzipPackage
    CheckCurIfCertPkg
    CheckPackageFull
}

CheckIfSanBoot()
{
    vmfsuuid=`ls -l / |grep altbootbank | awk -F" " '{print $NF}'`
    deviceinfo=`vmkfstools -P ${vmfsuuid} |tail -n 2 |head -n 1 |awk -F":" '{print $1}'`
    deviceinfo=$(echo ${deviceinfo})

    ExecuteCliRetString "${UPADMDIR}/upadm show vlun type=all"
    invalidInputNum=`echo "${TMPRETSTRING}" |grep -c "Invalid input"`
    if [ $invalidInputNum -gt 0 ]
    then
        typePara=""
    else
        typePara="type=all"
    fi

    ExecuteCliRetString "${UPADMDIR}/upadm show vlun ${typePara}"
    vlunInfo=`echo "${TMPRETSTRING}" |grep "^[[:blank:]]*[0-9]"|awk '{printf("%s ",$2)}'`
    if [ "${vlunInfo}" = "" ]
    then
        return 0
    fi

    for i in $vlunInfo
    do
        lunwwn=${i}
        if [ "${deviceinfo}" = "${lunwwn}" ]
        then
            return 1
        fi
    done
    
    return 0
}

if [ $1 ]
then
    case $1 in
    -ndu)
        CheckAttendInstallSystemInfo
        SetNewVersionAndOUSupport
        CheckOnlineUpgradeParameter $*
        InstallOnlineUpgrade
        ImportOem
        rm -rf /etc/cim/config/user_cfg.xml.old
        rm -rf $BAKDIR
        if [ -f $UPINSTALLLOCKFILE ]
        then
            rm -rf $UPINSTALLLOCKFILE
        fi
        exit 0
        ;;
    -f)
        OldUltraPthVersion=`esxcli software vib list | grep ultrapath.mpp | awk '{print $2}' | awk -F"-" '{print $1}'`
        if [ "${OldUltraPthVersion}" != "" ]
        then
            exit_with_msg "UltraPath already exists. The -f parameter cannot be used to update UltraPath."
        fi
        UnattendFlag=1
        CheckUnattendParameter $*
        ;;
    *)            
        exit_with_msg "The parameter value is incorrect." "main"
        ;;
    esac
fi

if [ ${UnattendFlag} -eq 1 ]
then    
    GetUattendConfFile
    
    CheckAttendInstallSystemInfo
    SetNewVersionAndOUSupport
    if [ ${waring_error_flag} -eq 1 ]
    then
        exit_with_msg "Unattend installation will exit. Rectify the preceding errors and perform installation again. "
        exit 1
    fi
else
    CheckAttendInstallSystemInfo
    SetNewVersionAndOUSupport
fi

CheckNXUP
if [ $? -eq 1 ]
then
    CheckLunMap
    if [ $? -eq 1 ]
    then
        CheckLunMapYesOrNo
    fi

    CheckClaimrule
    if [ $? -ne 0 ]
    then
        rm -rf $BAKDIR
        rm -rf $UPINSTALLLOCKFILE
        exit 1
    fi

    CheckWatchDog
    if [ $? -eq 0 ]
    then
        SFCBDNOTRUNINGFLAG=1
        TerminaLog_info "WARNING: The sfcbd-watchdog is not running." "main"
    fi
	
    if [ $CURPKG_IS_CERT_FLAG -eq 0 ]
    then
        esxcli software vib install -d $PARAMETER/$PKGNAME --no-sig-check
    else
        esxcli software vib install -d $PARAMETER/$PKGNAME
    fi
	
    if [ $? -ne 0 ]
    then
        TerminaLog_error "Excute esxcli software vib install -d $PARAMETER/$PKGNAME failed." "main"
        rm -rf $BAKDIR
        rm -rf $UPINSTALLLOCKFILE
        exit 1
    fi

    ImportOem
    cp -f $PARAMETER/$PKGNAME ${BACKUP_PKG_DIR}/

    TerminaLog_info "Please make sure your host name is uniqueness in data center." "main"
    
    VERSION=`vmware -v | awk '{print $3}'`
    if [ $SFCBDNOTRUNINGFLAG -ne 1 ]
    then
        if  [ "$VERSION" = "6.0.0" ]
        then
            /sbin/esxcfg-advcfg -A CIMvmw_ultrapath-providerProviderEnabled -T int -E "Enable or Disable the CIMvmw_ultrapath-providerProviderEnabled" -F 1 -N 0 -M 1
            esxcfg-advcfg --set 1 /UserVars/CIMvmw_ultrapath-providerProviderEnabled > /dev/null 2>&1
            /etc/init.d/sfcbd-watchdog restart > /dev/null 2>&1
        fi
		
        if [ "$VERSION" = "6.5.0" ]
        then
            /etc/init.d/sfcbd-watchdog restart > /dev/null 2>&1
        fi
    fi
    
    if [ $SFCBDNOTRUNINGFLAG -eq 1 ] && [ "$VERSION" != "6.5.0" ]
    then
        /sbin/esxcfg-advcfg -A CIMvmw_ultrapath-providerProviderEnabled -T int -E "Enable or Disable the CIMvmw_ultrapath-providerProviderEnabled" -F 1 -N 0 -M 1
        esxcfg-advcfg --set 1 /UserVars/CIMvmw_ultrapath-providerProviderEnabled  > /dev/null 2>&1
    fi

    if [ $NEEDBOOTFLAG -eq 1 ]
    then
        TerminaLog_info "WARNING: Some LUNs which should be managed by UltraPath are already managed by NMP or other third-party MPP." "main"
        TerminaLog_info "Please reboot system to make these LUNs managed by UltraPath, otherwise these LUNs may work abnormally." "main"
        if [ ${UnattendFlag} -eq 1 ]
        then
            echo  "Reboot system now? (n|y) [n] : ${UltraPathRestart}"
            Log_info "Reboot system now? (n|y) [n] : ${UltraPathRestart}" "main"
            if [ ${UltraPathRestart} = "Y" ] || [ ${UltraPathRestart} = "y" ]
            then
                echo "Unattend install configuration item 'restart' set to '${UltraPathRestart}'. The system will be restarted after the unattend installation."
                Log_info "Unattend install configuration item 'restart' set to '${UltraPathRestart}'. The system will be restarted after the unattend installation." "main"
                REBOOTFLAG=1
                reboot
            else
                echo "Unattend install configuration item 'restart' set to '${UltraPathRestart}'. The system will not be restarted after the unattend installation."
                Log_info "Unattend install configuration item 'restart' set to '${UltraPathRestart}'. The system will not be restarted after the unattend installation." "main"
                TerminaLog_info "Please reboot system later." "main"
            fi
            
        else
            RebootAfterUpgrade
        fi
    fi

    if [ $REBOOTFLAG -eq 1 ]
    then
        rm -rf $BAKDIR
        rm -rf $UPINSTALLLOCKFILE
        exit 0
    fi

    if [ $SFCBDNOTRUNINGFLAG -eq 1 ]
    then
        TerminaLog_info "The CIM Agent is not running,please restart the /etc/init.d/sfcbd-watchdog manually." "main"
    fi

    TerminaLog_info "Please restart the /etc/init.d/hostd manually." "main"

    rm -rf $UPINSTALLLOCKFILE
    rm -rf $BAKDIR
    
    exit 0
fi

CheckVersion
if [ $? -eq 1 ]
then
    rm -rf $UPINSTALLLOCKFILE
    rm -rf $BAKDIR
    exit 1
fi

CheckUpgradeClaimrule
if [ $? -eq 1 ]
then
    TerminaLog_error "Failed to upgrade ultrapath,exit." "main"
    rm -rf $UPINSTALLLOCKFILE
    rm -rf $BAKDIR
    exit 1
fi

ULTRAPATHCLI=`esxcli software vib list |grep ultrapath.cli`

if [ $OLDVERSION = $NEWVERSION ]
then
    ULTRAPATHCIM=`esxcli software vib list |grep ultrapath.cim`
    if [ "$ULTRAPATHCLI" != "" ] && [ "$ULTRAPATHCIM" != "" ]
    then
        TerminaLog_info "The UltraPath of the same version has already been installed." "main"
        rm -rf $BAKDIR
        rm -rf $UPINSTALLLOCKFILE
        exit 1
    fi
 
    if [ $CURPKG_IS_CERT_FLAG -eq 0 ]
    then
        esxcli software vib install -d $PARAMETER/$PKGNAME --no-sig-check
    else
        esxcli software vib install -d $PARAMETER/$PKGNAME
    fi
	
    if [ $? -ne 0 ]
    then
        TerminaLog_error "Excute esxcli software vib install -d $PARAMETER/$PKGNAME failed." "main"
        rm -rf $BAKDIR
        rm -rf $UPINSTALLLOCKFILE
        exit 1
    else
        ImportOem
        cp -f $PARAMETER/$PKGNAME ${BACKUP_PKG_DIR}/
        TerminaLog_info "Please restart the /etc/init.d/hostd manually." "main"
        rm -rf $UPINSTALLLOCKFILE
        rm -rf $BAKDIR
        exit 0
    fi
fi

if [ "$ULTRAPATHCLI" = "" ]
then
    TerminaLog_info "The ultrapath has been uninstalled,but the system needs to be rebooted for the changes to be effective." "main"
    TerminaLog_info "Please reboot your system." "main"
    rm -rf $UPINSTALLLOCKFILE
    rm -rf $BAKDIR
    exit 1
fi

TerminaLog_info "UltraPath upgrade may affect business. Please stop the business before upgrading the UltraPath." "main"
echo -n "* Are you sure that you want to continue the UltraPath upgrade? [y,n,?] : "
Log_info "* Are you sure that you want to continue the UltraPath upgrade? [y,n,?] : " "main"
while :
do
    ANSWER=""
    read ANSWER
    if [ "$ANSWER" = "Y" ] || [ "$ANSWER" = "y" ] || [ "$ANSWER" = "YES" ] || [ "$ANSWER" = "Yes" ] || [ "$ANSWER" = "yes" ]
    then
        Log_info "The answer is $ANSWER." "main"
        break
    elif [ "$ANSWER" = "N" ] || [ "$ANSWER" = "n" ] || [ "$ANSWER" = "NO" ] || [ "$ANSWER" = "No" ] || [ "$ANSWER" = "no" ]
    then
        Log_info "The answer is $ANSWER,exit." "main"
        rm -rf $UPINSTALLLOCKFILE
        rm -rf $BAKDIR
        exit 0
    else
        TerminaLog_error "        ERROR: Please enter yes or no." "main"
        echo -n "* Do you want to continue with the installation of <UltraPath> [y,n,?] : "
        Log_info "* Do you want to continue with the installation of <UltraPath> [y,n,?] : " "main"
    fi
done

BakOldFile
if [ $? -eq 1 ]
then
    rm -rf $UPINSTALLLOCKFILE
    rm -rf $BAKDIR
    exit 1
fi

CheckOfflineUpgrageUercfgXml

if [ $CURPKG_IS_CERT_FLAG -eq 0 ]
then
    esxcli software vib install -d $PARAMETER/$PKGNAME --no-sig-check > $TMPDIR/$INSTALLRST
else
    esxcli software vib install -d $PARAMETER/$PKGNAME > $TMPDIR/$INSTALLRST
fi

if [ $? -ne 0 ]
then
    exit_with_msg "The offline upgrade of UltraPath to ${NEW_VERSION} failed . Contact the technical support." "main"
fi

if [ -f $TMPDIR/$INSTALLRST ]
then
    SaveVLunInfo
    ISINSTALLSAME=`cat $TMPDIR/$INSTALLRST | grep -c "Host is not changed"`
    if [ $ISINSTALLSAME -eq 1 ]
    then
        OfflineUpgradeNotReboot
        ImportOem
        cp -f $PARAMETER/$PKGNAME ${BACKUP_PKG_DIR}/
        if [ $NEEDBOOTFLAG -ne 0 ]
        then
            TerminaLog_info "The update completed successfully, but the system needs to be rebooted for the changes to be effective." "main"
        else
            TerminaLog_info "The update completed successfully." "main"
            TerminaLog_info "Note: The output of the esxcli software vib list command shows that the UltraPath version is the pre-upgrade one. Run the esxcli upadm show version command to check the version of the current UltraPath." "main"
        fi
        chmodExec 600 $CONDIR/$UPCONFIG

        rm -rf /tmp/.vlunname
        rm -rf $BAKDIR
        rm -rf $UPINSTALLLOCKFILE
        rm -rf $TMPDIR/$INSTALLRST
        rm -rf $CONDIR/$UPCONFIGOLD
        if [ $NEEDBOOTFLAG -ne 0 ]
        then
            RebootAfterUpgrade
        fi
        exit 0
    fi

    cat $TMPDIR/$INSTALLRST | grep "The update completed successfully" >> "${LOGFILE}" 2>&1
    if [ $? -eq 0 ]
    then
        OfflineUpgradeNotReboot
        CopyFileToNewPkg
        chmodExec 700 $BAKDIR/cfg_upgrade

        $BAKDIR/cfg_upgrade -r8
        if [ $? -eq 0 ]
        then
            ImportOem
            rm -rf ${BACKUP_PKG_DIR}/*
            cp -f $PARAMETER/$PKGNAME ${BACKUP_PKG_DIR}/
            if [ $NEEDBOOTFLAG -ne 0 ]
            then
                TerminaLog_info "The update completed successfully, but the system needs to be rebooted for the changes to be effective." "main"
            else
                TerminaLog_info "The update completed successfully." "main"
                TerminaLog_info "Note: The output of the esxcli software vib list command shows that the UltraPath version is the pre-upgrade one. Run the esxcli upadm show version command to check the version of the current UltraPath." "main"
            fi          
        else
            TerminaLog_error "The update completed failed." "main"
            cp -rf $CONDIR/$UPCONFIGOLD $CONDIR/$UPCONFIG
        fi
        
        chmodExec 600 $CONDIR/$UPCONFIG

        rm -rf /tmp/.vlunname
        rm -rf $BAKDIR
        rm -rf $UPINSTALLLOCKFILE
        rm -rf $TMPDIR/$INSTALLRST
        rm -rf $CONDIR/$UPCONFIGOLD
        if [ $NEEDBOOTFLAG -ne 0 ]
        then
            RebootAfterUpgrade
        fi
        exit 0
    fi

    TerminaLog_error "Some unkown error happened." "main"
    rm -rf /tmp/.vlunname
    rm -rf $BAKDIR
    rm -rf $UPINSTALLLOCKFILE
    rm -rf $TMPDIR/$INSTALLRST
    exit 1  
else
    TerminaLog_error "The install result file is not exist." "main"
    rm -rf $BAKDIR
    rm -rf $UPINSTALLLOCKFILE
    rm -rf $TMPDIR/$INSTALLRST
    exit 1
fi
